@@ -1,8 +1,9 @@
-/* $Id: chars.c,v 1.94 2005/09/14 20:20:58 dolorous Exp $ */
+/* $Id: chars.c,v 1.108 2005/12/08 02:47:10 dolorous Exp $ */
 /**************************************************************************
  *   chars.c                                                              *
  *                                                                        *
- *   Copyright (C) 2005 Chris Allegretta                                  *
+ *   Copyright (C) 2001-2004 Chris Allegretta                             *
+ *   Copyright (C) 2005 David Lawrence Ramsey                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -20,13 +21,10 @@
  *                                                                        *
  **************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "proto.h"
 
 #include <string.h>
 #include <ctype.h>
-#include "proto.h"
 
 #ifdef ENABLE_UTF8
 #ifdef HAVE_WCHAR_H
@@ -299,8 +297,10 @@ int mbwidth(const char *c)
 
 	width = wcwidth(wc);
 
-	if (width == -1)
-	    width++;
+	if (width == -1) {
+	    wc = bad_wchar;
+	    width = wcwidth(wc);
+	}
 
 	return width;
     } else
@@ -365,8 +365,8 @@ int parse_mbchar(const char *buf, char *chr, size_t *col)
 	/* Get the number of bytes in the multibyte character. */
 	buf_mb_len = mblen(buf, MB_CUR_MAX);
 
-	/* If buf contains an invalid multibyte character, set bad_chr
-	 * to TRUE and interpret buf's first byte. */
+	/* If buf contains an invalid multibyte character, only
+	 * interpret buf's first byte. */
 	if (buf_mb_len < 0) {
 	    mblen(NULL, 0);
 	    buf_mb_len = 1;
@@ -452,7 +452,7 @@ size_t move_mbleft(const char *buf, size_t pos)
     while (TRUE) {
 	int buf_mb_len = parse_mbchar(buf + pos - pos_prev, NULL, NULL);
 
-	if (pos_prev <= (size_t)buf_mb_len)
+	if (pos_prev <= buf_mb_len)
 	    break;
 
 	pos_prev -= buf_mb_len;
@@ -551,8 +551,31 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 }
 
 #ifndef HAVE_STRCASESTR
-/* This function is equivalent to strcasestr().  It was adapted from
- * mutt's mutt_stristr() function. */
+/* This function, nstrcasestr() (originally mutt_stristr()), was adapted
+ * from mutt 1.2.4i (lib.c).  Here is the notice from that file, with
+ * the Free Software Foundation's address updated:
+ *
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1999-2000 Thomas Roessler <roessler@guug.de>
+ * 
+ *     This program is free software; you can redistribute it
+ *     and/or modify it under the terms of the GNU General Public
+ *     License as published by the Free Software Foundation; either
+ *     version 2 of the License, or (at your option) any later
+ *     version.
+ * 
+ *     This program is distributed in the hope that it will be
+ *     useful, but WITHOUT ANY WARRANTY; without even the implied
+ *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+ *     PURPOSE.  See the GNU General Public License for more
+ *     details.
+ * 
+ *     You should have received a copy of the GNU General Public
+ *     License along with this program; if not, write to the Free
+ *     Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ *     Boston, MA  02110-1301, USA. */
+
+/* This function is equivalent to strcasestr(). */
 const char *nstrcasestr(const char *haystack, const char *needle)
 {
     assert(haystack != NULL && needle != NULL);
@@ -631,7 +654,7 @@ const char *mbstrcasestr(const char *haystack, const char *needle)
 	return strcasestr(haystack, needle);
 }
 
-#if !defined(NANO_SMALL) || !defined(DISABLE_TABCOMP)
+#if !defined(NANO_TINY) || !defined(DISABLE_TABCOMP)
 /* This function is equivalent to strstr(), except in that it scans the
  * string in reverse, starting at rev_start. */
 const char *revstrstr(const char *haystack, const char *needle, const
@@ -651,9 +674,9 @@ const char *revstrstr(const char *haystack, const char *needle, const
 
     return NULL;
 }
-#endif /* !NANO_SMALL || !DISABLE_TABCOMP */
+#endif /* !NANO_TINY || !DISABLE_TABCOMP */
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 /* This function is equivalent to strcasestr(), except in that it scans
  * the string in reverse, starting at rev_start. */
 const char *revstrcasestr(const char *haystack, const char *needle,
@@ -740,7 +763,7 @@ const char *mbrevstrcasestr(const char *haystack, const char *needle,
 #endif
 	return revstrcasestr(haystack, needle, rev_start);
 }
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
 
 /* This function is equivalent to strlen() for multibyte strings. */
 size_t mbstrlen(const char *s)
@@ -790,6 +813,26 @@ size_t mbstrnlen(const char *s, size_t maxlen)
 	return strnlen(s, maxlen);
 }
 
+#ifndef NANO_TINY
+/* This function is equivalent to strpbrk(), except in that it scans the
+ * string in reverse, starting at rev_start. */
+char *revstrpbrk(const char *s, const char *accept, const char
+	*rev_start)
+{
+    assert(s != NULL && accept != NULL && rev_start != NULL);
+
+    for (; rev_start >= s; rev_start--) {
+	const char *q = (*rev_start == '\0') ? NULL : strchr(accept,
+		*rev_start);
+
+	if (q != NULL)
+	    return (char *)rev_start;
+    }
+
+    return NULL;
+}
+#endif /* !NANO_TINY */
+
 #ifndef DISABLE_JUSTIFY
 /* This function is equivalent to strchr() for multibyte strings. */
 char *mbstrchr(const char *s, char *c)
@@ -798,7 +841,7 @@ char *mbstrchr(const char *s, char *c)
 
 #ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
-	bool bad_c_mb = FALSE, bad_s_mb = FALSE;
+	bool bad_s_mb = FALSE, bad_c_mb = FALSE;
 	char *s_mb = charalloc(MB_CUR_MAX);
 	const char *q = s;
 	wchar_t ws, wc;
@@ -828,7 +871,7 @@ char *mbstrchr(const char *s, char *c)
 
 	free(s_mb);
 
-	if (ws != wc)
+	if (*s == '\0')
 	    q = NULL;
 
 	return (char *)q;
