@@ -1,8 +1,9 @@
-/* $Id: color.c,v 1.33 2005/08/01 05:54:11 dolorous Exp $ */
+/* $Id: color.c,v 1.37 2005/12/08 02:47:10 dolorous Exp $ */
 /**************************************************************************
  *   color.c                                                              *
  *                                                                        *
- *   Copyright (C) 1999-2005 Chris Allegretta                             *
+ *   Copyright (C) 2001-2004 Chris Allegretta                             *
+ *   Copyright (C) 2005 David Lawrence Ramsey                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -20,18 +21,15 @@
  *                                                                        *
  **************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "proto.h"
 
 #include <stdio.h>
 #include <string.h>
-#include "proto.h"
 
 #ifdef ENABLE_COLOR
 
-/* For each syntax list entry, we go through the list of colors and
- * assign color pairs. */
+/* For each syntax list entry, go through the list of colors and assign
+ * color pairs. */
 void set_colorpairs(void)
 {
     const syntaxtype *this_syntax = syntaxes;
