@@ -1,8 +1,9 @@
-/* $Id: cut.c,v 1.32 2005/07/24 19:57:51 dolorous Exp $ */
+/* $Id: cut.c,v 1.44 2005/12/08 07:09:08 dolorous Exp $ */
 /**************************************************************************
  *   cut.c                                                                *
  *                                                                        *
- *   Copyright (C) 1999-2005 Chris Allegretta                             *
+ *   Copyright (C) 1999-2004 Chris Allegretta                             *
+ *   Copyright (C) 2005 David Lawrence Ramsey                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -20,37 +21,40 @@
  *                                                                        *
  **************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "proto.h"
 
 #include <string.h>
 #include <stdio.h>
-#include "proto.h"
 
 static bool keep_cutbuffer = FALSE;
 	/* Should we keep the contents of the cutbuffer? */
 static filestruct *cutbottom = NULL;
-	/* Pointer to end of cutbuffer. */
+	/* Pointer to the end of the cutbuffer. */
 
+/* Indicate that we should no longer keep the contents of the
+ * cutbuffer. */
 void cutbuffer_reset(void)
 {
     keep_cutbuffer = FALSE;
 }
 
-/* If we're not on the magicline, move all the text of the current line,
- * plus the newline at the end, to the cutbuffer, and set the current
- * place we want to where the line used to start. */
+/* If we're not on the last line of the file, move all the text of the
+ * current line, plus the newline at the end, to the cutbuffer.  If we
+ * are, move all of the text of the current line to the cutbuffer.  In
+ * both cases, set the current place we want to the beginning of the
+ * current line. */
 void cut_line(void)
 {
-    if (openfile->current->next != NULL) {
+    if (openfile->current != openfile->filebot)
 	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current, 0,
 		openfile->current->next, 0);
-	openfile->placewewant = xplustabs();
-    }
+    else
+	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current, 0,
+		openfile->current, strlen(openfile->current->data));
+    openfile->placewewant = 0;
 }
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 /* Move all currently marked text to the cutbuffer, and set the current
  * place we want to where the text used to start. */
 void cut_marked(void)
@@ -68,9 +72,9 @@ void cut_marked(void)
 /* If we're not at the end of the current line, move all the text from
  * the current cursor position to the end of the current line,
  * not counting the newline at the end, to the cutbuffer.  If we are,
- * and we're not on the magicline, move the newline at the end to the
- * cutbuffer, and set the current place we want to where the newline
- * used to be. */
+ * and we're not on the last line of the file, move the newline at the
+ * end to the cutbuffer, and set the current place we want to where the
+ * newline used to be. */
 void cut_to_eol(void)
 {
     size_t data_len = strlen(openfile->current->data);
@@ -83,17 +87,17 @@ void cut_to_eol(void)
 	 * the end, to the cutbuffer. */
 	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current,
 		openfile->current_x, openfile->current, data_len);
-    else if (openfile->current->next != NULL) {
-	/* If we're at the end of the line, and it isn't the magicline,
-	 * move all the text from the current position up to the
-	 * beginning of the next line, i.e, the newline at the end, to
-	 * the cutbuffer. */
+    else if (openfile->current != openfile->filebot) {
+	/* If we're at the end of the line, and it isn't the last line
+	 * of the file, move all the text from the current position up
+	 * to the beginning of the next line, i.e, the newline at the
+	 * end, to the cutbuffer. */
 	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current,
 		openfile->current_x, openfile->current->next, 0);
 	openfile->placewewant = xplustabs();
     }
 }
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
 
 /* Move text from the current filestruct into the cutbuffer. */
 void do_cut_text(void)
@@ -117,7 +121,7 @@ void do_cut_text(void)
      * cutbuffer instead of replacing it. */
     keep_cutbuffer = TRUE;
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     if (openfile->mark_set) {
 	/* If the mark is on, move the marked text to the cutbuffer and
 	 * turn the mark off. */
@@ -140,7 +144,7 @@ void do_cut_text(void)
 #endif
 }
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 /* Cut from the current cursor position to the end of the file. */
 void do_cut_till_end(void)
 {
@@ -149,7 +153,8 @@ void do_cut_till_end(void)
     check_statusblank();
 
     move_to_filestruct(&cutbuffer, &cutbottom, openfile->current,
-	openfile->current_x, openfile->filebot, 0);
+	openfile->current_x, openfile->filebot,
+	strlen(openfile->filebot->data));
 
     edit_refresh();
     set_modified();
@@ -158,7 +163,7 @@ void do_cut_till_end(void)
     dump_filestruct(cutbuffer);
 #endif
 }
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
 
 /* Copy text from the cutbuffer into the current filestruct. */
 void do_uncut_text(void)
