@@ -1,8 +1,9 @@
-/* $Id: files.c,v 1.247 2005/08/29 19:11:26 dolorous Exp $ */
+/* $Id: files.c,v 1.270 2005/12/08 07:24:54 dolorous Exp $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
- *   Copyright (C) 1999-2005 Chris Allegretta                             *
+ *   Copyright (C) 1999-2004 Chris Allegretta                             *
+ *   Copyright (C) 2005 David Lawrence Ramsey                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -20,20 +21,16 @@
  *                                                                        *
  **************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "proto.h"
 
-#include <string.h>
 #include <stdio.h>
+#include <string.h>
 #include <unistd.h>
-#include <sys/wait.h>
 #include <utime.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <ctype.h>
 #include <pwd.h>
-#include "proto.h"
 
 /* Add an entry to the openfile openfilestruct.  This should only be
  * called from open_buffer(). */
@@ -69,7 +66,7 @@ void initialize_buffer(void)
     openfile->current_y = 0;
 
     openfile->modified = FALSE;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     openfile->mark_set = FALSE;
 
     openfile->mark_begin = NULL;
@@ -144,7 +141,7 @@ void open_buffer(const char *filename)
      * no stat, update the stat, if applicable. */
     if (rc == 0) {
 	read_file(f, filename);
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	if (openfile->current_stat == NULL) {
 	    openfile->current_stat =
 		(struct stat *)nmalloc(sizeof(struct stat));
@@ -154,9 +151,12 @@ void open_buffer(const char *filename)
     }
 
     /* If we have a file and we're loading into a new buffer, move back
-     * to the first line of the buffer. */
-    if (rc != -1 && new_buffer)
+     * to the beginning of the first line of the buffer. */
+    if (rc != -1 && new_buffer) {
 	openfile->current = openfile->fileage;
+	openfile->current_x = 0;
+	openfile->placewewant = 0;
+    }
 
 #ifdef ENABLE_COLOR
     /* If we're loading into a new buffer, update the colors to account
@@ -166,6 +166,39 @@ void open_buffer(const char *filename)
 #endif
 }
 
+#ifndef DISABLE_SPELLER
+/* If it's not "", filename is a file to open.  We blow away the text of
+ * the current buffer, and then open and read the file, if
+ * applicable.  Note that we skip the operating directory test when
+ * doing this. */
+void replace_buffer(const char *filename)
+{
+    FILE *f;
+    int rc;
+	/* rc == -2 means that we have a new file.  -1 means that the
+	 * open() failed.  0 means that the open() succeeded. */
+
+    assert(filename != NULL);
+
+    /* If the filename isn't blank, open the file.  Otherwise, treat it
+     * as a new file. */
+    rc = (filename[0] != '\0') ? open_file(filename, TRUE, &f) : -2;
+
+    /* Reinitialize the text of the current buffer. */
+    free_filestruct(openfile->fileage);
+    initialize_buffer_text();
+
+    /* If we have a non-new file, read it in. */
+    if (rc == 0)
+	read_file(f, filename);
+
+    /* Move back to the beginning of the first line of the buffer. */
+    openfile->current = openfile->fileage;
+    openfile->current_x = 0;
+    openfile->placewewant = 0;
+}
+#endif /* !DISABLE_SPELLER */
+
 /* Update the screen to account for the current buffer. */
 void display_buffer(void)
 {
@@ -269,7 +302,7 @@ filestruct *read_line(char *buf, filestruct *prevnode, bool
 
     fileptr->data = mallocstrcpy(NULL, buf);
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     /* If it's a DOS file ("\r\n"), and file conversion isn't disabled,
      * strip the '\r' part from fileptr->data. */
     if (!ISSET(NO_CONVERT) && buf_len > 0 && buf[buf_len - 1] == '\r')
@@ -303,6 +336,8 @@ filestruct *read_line(char *buf, filestruct *prevnode, bool
     return fileptr;
 }
 
+/* Read an open file into the current buffer.  f should be set to the
+ * open file, and filename should be set to the name of the file. */
 void read_file(FILE *f, const char *filename)
 {
     size_t num_lines = 0;
@@ -324,7 +359,7 @@ void read_file(FILE *f, const char *filename)
     int input_int;
 	/* The current value we read from the file, whether an input
 	 * character or EOF. */
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     int format = 0;
 	/* 0 = *nix, 1 = DOS, 2 = Mac, 3 = both DOS and Mac. */
 #endif
@@ -346,7 +381,7 @@ void read_file(FILE *f, const char *filename)
 	/* If it's a *nix file ("\n") or a DOS file ("\r\n"), and file
 	 * conversion isn't disabled, handle it! */
 	if (input == '\n') {
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    /* If there's a '\r' before the '\n', set format to DOS if
 	     * we currently think this is a *nix file, or to both if we
 	     * currently think it's a Mac file. */
@@ -365,7 +400,7 @@ void read_file(FILE *f, const char *filename)
 	    num_lines++;
 	    buf[0] = '\0';
 	    i = 0;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	/* If it's a Mac file ('\r' without '\n'), and file conversion
 	 * isn't disabled, handle it! */
 	} else if (!ISSET(NO_CONVERT) && i > 0 && buf[i - 1] == '\r') {
@@ -414,7 +449,7 @@ void read_file(FILE *f, const char *filename)
 	nperror(filename);
     fclose(f);
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     /* If file conversion isn't disabled and the last character in this
      * file is '\r', read it in properly as a Mac format line. */
     if (len == 0 && !ISSET(NO_CONVERT) && input == '\r') {
@@ -427,7 +462,7 @@ void read_file(FILE *f, const char *filename)
 
     /* Did we not get a newline and still have stuff to do? */
     if (len > 0) {
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	/* If file conversion isn't disabled and the last character in
 	 * this file is '\r', set format to Mac if we currently think
 	 * the file is a *nix file, or to both DOS and Mac if we
@@ -449,25 +484,76 @@ void read_file(FILE *f, const char *filename)
     if (fileptr == NULL)
 	open_buffer("");
 
-    /* Did we try to insert a file of zero bytes? */
-    if (num_lines != 0) {
-	if (openfile->current != NULL) {
+    /* Attach the file we got to the filestruct.  If we got a file of
+     * zero bytes, don't do anything. */
+    if (num_lines > 0) {
+	/* If the file we got doesn't end in a newline, tack its last
+	 * line onto the beginning of the line at current. */
+	if (len > 0) {
+	    size_t current_len = strlen(openfile->current->data);
+
+	    /* Adjust the current x-coordinate to compensate for the
+	     * change in the current line. */
+	    if (num_lines == 1)
+		openfile->current_x += len;
+	    else
+		openfile->current_x = len;
+
+	    /* Tack the text at fileptr onto the beginning of the text
+	     * at current. */
+	    openfile->current->data =
+		charealloc(openfile->current->data, len +
+		current_len + 1);
+	    charmove(openfile->current->data + len,
+		openfile->current->data, current_len + 1);
+	    strncpy(openfile->current->data, fileptr->data, len);
+
+	    /* Don't destroy fileage, edittop, or filebot! */
+	    if (fileptr == openfile->fileage)
+		openfile->fileage = openfile->current;
+	    if (fileptr == openfile->edittop)
+		openfile->edittop = openfile->current;
+	    if (fileptr == openfile->filebot)
+		openfile->filebot = openfile->current;
+
+	    /* Move fileptr back one line and blow away the old fileptr,
+	     * since its text has been saved. */
+	    fileptr = fileptr->prev;
+	    if (fileptr != NULL) {
+		if (fileptr->next != NULL)
+		    free(fileptr->next);
+	    }
+	}
+
+	/* Attach the line at current after the line at fileptr. */
+	if (fileptr != NULL) {
 	    fileptr->next = openfile->current;
 	    openfile->current->prev = fileptr;
-	    renumber(openfile->current);
-	    openfile->current_x = 0;
-	    openfile->placewewant = 0;
-	} else if (fileptr->next == NULL) {
-	    openfile->filebot = fileptr;
-	    new_magicline();
-	    openfile->totsize--;
 	}
+
+	/* Renumber starting with the last line of the file we
+	 * inserted. */
+	renumber(openfile->current);
     }
 
     openfile->totsize += get_totsize(openfile->fileage,
 	openfile->filebot);
 
-#ifndef NANO_SMALL
+    /* If the NO_NEWLINES flag isn't set, and text has been added to
+     * the magicline (i.e, a file that doesn't end in a newline has been
+     * inserted at the end of the current buffer), add a new magicline,
+     * and move the current line down to it. */
+    if (!ISSET(NO_NEWLINES) && openfile->filebot->data[0] != '\0') {
+	new_magicline();
+	openfile->current = openfile->filebot;
+	openfile->current_x = 0;
+    }
+
+    /* Set the current place we want to the end of the last line of the
+     * file we inserted. */
+    openfile->placewewant = xplustabs();
+
+#ifndef NANO_TINY
     if (format == 3)
 	statusbar(
 		P_("Read %lu line (Converted from DOS and Mac format)",
@@ -509,6 +595,7 @@ int open_file(const char *filename, bool newfie, FILE **f)
 	    return -2;
 	}
 	statusbar(_("\"%s\" not found"), filename);
+	beep();
 	return -1;
     } else if (S_ISDIR(fileinfo.st_mode) || S_ISCHR(fileinfo.st_mode) ||
 	S_ISBLK(fileinfo.st_mode)) {
@@ -516,9 +603,11 @@ int open_file(const char *filename, bool newfie, FILE **f)
 	statusbar(S_ISDIR(fileinfo.st_mode) ?
 		_("\"%s\" is a directory") :
 		_("File \"%s\" is a device file"), filename);
+	beep();
 	return -1;
     } else if ((fd = open(filename, O_RDONLY)) == -1) {
 	statusbar(_("Error reading %s: %s"), filename, strerror(errno));
+	beep();
  	return -1;
      } else {
 	/* File is A-OK.  Open it in binary mode for our own end-of-line
@@ -528,6 +617,7 @@ int open_file(const char *filename, bool newfie, FILE **f)
 	if (*f == NULL) {
 	    statusbar(_("Error reading %s: %s"), filename,
 		strerror(errno));
+	    beep();
 	    close(fd);
 	} else
 	    statusbar(_("Reading File"));
@@ -542,16 +632,20 @@ int open_file(const char *filename, bool newfie, FILE **f)
  * extension exists, we return "". */
 char *get_next_filename(const char *name, const char *suffix)
 {
+    static int ulmax_digits = -1;
     unsigned long i = 0;
     char *buf;
     size_t namelen, suffixlen;
 
     assert(name != NULL && suffix != NULL);
 
+    if (ulmax_digits == -1)
+	ulmax_digits = digits(ULONG_MAX);
+
     namelen = strlen(name);
     suffixlen = strlen(suffix);
 
-    buf = charalloc(namelen + suffixlen + digits(ULONG_MAX) + 2);
+    buf = charalloc(namelen + suffixlen + ulmax_digits + 2);
     sprintf(buf, "%s%s", name, suffix);
 
     while (TRUE) {
@@ -573,8 +667,11 @@ char *get_next_filename(const char *name, const char *suffix)
     return buf;
 }
 
+/* Insert a file into a new buffer if the MULTIBUFFER flag is set, or
+ * into the current buffer if it isn't.  If execute is TRUE, insert the
+ * output of an executed command instead of a file. */
 void do_insertfile(
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	bool execute
 #else
 	void
@@ -595,7 +692,7 @@ void do_insertfile(
 #endif
 
     while (TRUE) {
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	if (execute) {
 	    msg = 
 #ifdef ENABLE_MULTIBUFFER
@@ -611,16 +708,16 @@ void do_insertfile(
 		N_("File to insert into new buffer [from %s] ") :
 #endif
 		N_("File to insert [from %s] ");
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	}
 #endif
 
-	i = statusq(TRUE,
-#ifndef NANO_SMALL
+	i = do_prompt(TRUE,
+#ifndef NANO_TINY
 		execute ? extcmd_list :
 #endif
 		insertfile_list, ans,
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 		NULL,
 #endif
 		_(msg),
@@ -644,7 +741,7 @@ void do_insertfile(
 
 	    ans = mallocstrcpy(ans, answer);
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 #ifdef ENABLE_MULTIBUFFER
 	    if (i == TOGGLE_MULTIBUFFER_KEY) {
 		/* Don't allow toggling if we're in view mode. */
@@ -660,7 +757,7 @@ void do_insertfile(
 #ifndef DISABLE_BROWSER
 	    else
 #endif
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
 
 #ifndef DISABLE_BROWSER
 	    if (i == NANO_TOFILES_KEY) {
@@ -675,7 +772,7 @@ void do_insertfile(
 		/* We have a file now.  Indicate this and get out of the
 		 * statusbar prompt cleanly. */
 		i = 0;
-		statusq_abort();
+		do_prompt_abort();
 	    }
 #endif
 
@@ -705,7 +802,7 @@ void do_insertfile(
 	    }
 #endif
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    if (execute) {
 #ifdef ENABLE_MULTIBUFFER
 		if (ISSET(MULTIBUFFER))
@@ -725,7 +822,7 @@ void do_insertfile(
 		/* Save the file specified in answer in the current
 		 * buffer. */
 		open_buffer(answer);
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    }
 #endif
 
@@ -779,6 +876,9 @@ void do_insertfile(
     free(ans);
 }
 
+/* Insert a file into a new buffer or the current buffer, depending on
+ * whether the MULTIBUFFER flag is set.  If we're in view mode, only
+ * allow inserting a file into a new buffer. */
 void do_insertfile_void(void)
 {
 #ifdef ENABLE_MULTIBUFFER
@@ -787,7 +887,7 @@ void do_insertfile_void(void)
     else
 #endif
 	do_insertfile(
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 		FALSE
 #endif
 		);
@@ -1066,7 +1166,7 @@ bool check_operating_dir(const char *currpath, bool allow_tabcomp)
 }
 #endif
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 void init_backup_dir(void)
 {
     char *full_backup_dir;
@@ -1207,7 +1307,7 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
      * one). */
     realexists = (stat(realname, &st) != -1);
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     /* We backup only if the backup toggle is set, the file isn't
      * temporary, and the file already exists.  Furthermore, if we
      * aren't appending, prepending, or writing a selection, we backup
@@ -1232,6 +1332,7 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 	    if (f == NULL) {
 		statusbar(_("Error reading %s: %s"), realname,
 			strerror(errno));
+		beep();
 		goto cleanup_and_exit;
 	    }
 	}
@@ -1311,10 +1412,11 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 		openfile->current_stat->st_gid) == -1 ||
 		utime(backupname, &filetime) == -1) {
 	    free(backupname);
-	    if (copy_status == -1)
+	    if (copy_status == -1) {
 		statusbar(_("Error reading %s: %s"), realname,
 			strerror(errno));
-	    else
+		beep();
+	    } else
 		statusbar(_("Error writing %s: %s"), backupname,
 			strerror(errno));
 	    goto cleanup_and_exit;
@@ -1322,7 +1424,7 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 
 	free(backupname);
     }
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
 
     /* If NOFOLLOW_SYMLINKS is set and the file is a link, we aren't
      * doing prepend or append.  So we delete the link first, and just
@@ -1365,6 +1467,7 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 		if (f_source == NULL) {
 		    statusbar(_("Error reading %s: %s"), realname,
 			strerror(errno));
+		    beep();
 		    close(fd_source);
 		    fclose(f);
 		    unlink(tempname);
@@ -1384,10 +1487,9 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
     if (f_open == NULL) {
 	/* Now open the file in place.  Use O_EXCL if tmp is TRUE.  This
 	 * is copied from joe, because wiggy says so *shrug*. */
-	fd = open(realname, O_WRONLY | O_CREAT |
-		((append == APPEND) ? O_APPEND : (tmp ? O_EXCL :
-		O_TRUNC)), S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP |
-		S_IROTH | S_IWOTH);
+	fd = open(realname, O_WRONLY | O_CREAT | ((append == APPEND) ?
+		O_APPEND : (tmp ? O_EXCL : O_TRUNC)), S_IRUSR |
+		S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
 
 	/* Set the umask back to the user's original value. */
 	umask(original_umask);
@@ -1417,7 +1519,7 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
      * a selection. */
     assert(openfile->fileage != NULL && openfile->filebot != NULL);
 
-    while (fileptr != openfile->filebot) {
+    while (fileptr != NULL) {
 	size_t data_len = strlen(fileptr->data), size;
 
 	/* Newlines to nulls, just before we write to disk. */
@@ -1435,27 +1537,37 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 	    goto cleanup_and_exit;
 	}
 
-#ifndef NANO_SMALL
-	if (openfile->fmt == DOS_FILE || openfile->fmt == MAC_FILE) {
-	    if (putc('\r', f) == EOF) {
-		statusbar(_("Error writing %s: %s"), realname,
+	/* If we're on the last line of the file, don't write a newline
+	 * character after it.  If the last line of the file is blank,
+	 * this means that zero bytes are written, in which case we
+	 * don't count the last line in the total lines written. */
+	if (fileptr == openfile->filebot) {
+	    if (fileptr->data[0] == '\0')
+		lineswritten--;
+	} else {
+#ifndef NANO_TINY
+	    if (openfile->fmt == DOS_FILE || openfile->fmt ==
+		MAC_FILE) {
+		if (putc('\r', f) == EOF) {
+		    statusbar(_("Error writing %s: %s"), realname,
 			strerror(errno));
-		fclose(f);
-		goto cleanup_and_exit;
+		    fclose(f);
+		    goto cleanup_and_exit;
+		}
 	    }
-	}
 
-	if (openfile->fmt != MAC_FILE) {
+	    if (openfile->fmt != MAC_FILE) {
 #endif
-	    if (putc('\n', f) == EOF) {
-		statusbar(_("Error writing %s: %s"), realname,
+		if (putc('\n', f) == EOF) {
+		    statusbar(_("Error writing %s: %s"), realname,
 			strerror(errno));
-		fclose(f);
-		goto cleanup_and_exit;
+		    fclose(f);
+		    goto cleanup_and_exit;
+		}
+#ifndef NANO_TINY
 	    }
-#ifndef NANO_SMALL
-	}
 #endif
+	}
 
 	fileptr = fileptr->next;
 	lineswritten++;
@@ -1477,6 +1589,7 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 	if (f_source == NULL) {
 	    statusbar(_("Error reading %s: %s"), tempname,
 		strerror(errno));
+	    beep();
 	    fclose(f);
 	    goto cleanup_and_exit;
 	}
@@ -1511,7 +1624,7 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 #endif
 	}
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	/* Update current_stat to reference the file as it is now. */
 	if (openfile->current_stat == NULL)
 	    openfile->current_stat =
@@ -1535,7 +1648,7 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
     return retval;
 }
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 /* Write a marked selection from a file out. */
 int write_marked_file(const char *name, FILE *f_open, bool tmp,
 	append_type append)
@@ -1543,7 +1656,7 @@ int write_marked_file(const char *name, FILE *f_open, bool tmp,
     int retval = -1;
     bool old_modified = openfile->modified;
 	/* write_file() unsets the modified flag. */
-    bool added_magicline;
+    bool added_magicline = FALSE;
 	/* Whether we added a magicline after filebot. */
     filestruct *top, *bot;
     size_t top_x, bot_x;
@@ -1556,17 +1669,19 @@ int write_marked_file(const char *name, FILE *f_open, bool tmp,
 	(const filestruct **)&bot, &bot_x, NULL);
     filepart = partition_filestruct(top, top_x, bot, bot_x);
 
-    /* If the line at filebot is blank, treat it as the magicline and
-     * hence the end of the file.  Otherwise, add a magicline and treat
-     * it as the end of the file. */
-    added_magicline = (openfile->filebot->data[0] != '\0');
-    if (added_magicline)
+    /* Handle the magicline if the NO_NEWLINES flag isn't set.  If the
+     * line at filebot is blank, treat it as the magicline and hence the
+     * end of the file.  Otherwise, add a magicline and treat it as the
+     * end of the file. */
+    if (!ISSET(NO_NEWLINES) &&
+	(added_magicline = (openfile->filebot->data[0] != '\0')))
 	new_magicline();
 
     retval = write_file(name, f_open, tmp, append, TRUE);
 
-    /* If we added a magicline, remove it now. */
-    if (added_magicline)
+    /* If the NO_NEWLINES flag isn't set, and we added a magicline,
+     * remove it now. */
+    if (!ISSET(NO_NEWLINES) && added_magicline)
 	remove_magicline();
 
     /* Unpartition the filestruct so that it contains all the text
@@ -1578,8 +1693,12 @@ int write_marked_file(const char *name, FILE *f_open, bool tmp,
 
     return retval;
 }
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
 
+/* Write the current file to disk.  If the mark is on, write the current
+ * marked selection to disk.  If exiting is TRUE, write the file to disk
+ * regardless of whether the mark is on, and without prompting if the
+ * TEMP_FILE flag is set. */
 int do_writeout(bool exiting)
 {
     int i, retval = 0;
@@ -1602,14 +1721,14 @@ int do_writeout(bool exiting)
     }
 
     ans = mallocstrcpy(NULL,
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	(openfile->mark_set && !exiting) ? "" :
 #endif
 	openfile->filename);
 
     while (TRUE) {
 	const char *msg;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	const char *formatstr, *backupstr;
 
 	formatstr = (openfile->fmt == DOS_FILE) ?
@@ -1624,7 +1743,7 @@ int do_writeout(bool exiting)
 		N_("Append Selection to File") :
 		N_("Write Selection to File");
 	else
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
 	    msg = (append == PREPEND) ? N_("File Name to Prepend to") :
 		(append == APPEND) ? N_("File Name to Append to") :
 		N_("File Name to Write");
@@ -1632,9 +1751,9 @@ int do_writeout(bool exiting)
 	/* If we're using restricted mode, the filename isn't blank,
 	 * and we're at the "Write File" prompt, disable tab
 	 * completion. */
-	i = statusq(!ISSET(RESTRICTED) ||
+	i = do_prompt(!ISSET(RESTRICTED) ||
 		openfile->filename[0] == '\0', writefile_list, ans,
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 		NULL, "%s%s%s", _(msg), formatstr, backupstr
 #else
 		"%s", _(msg)
@@ -1662,10 +1781,10 @@ int do_writeout(bool exiting)
 
 		/* We have a file now.  Get out of the statusbar prompt
 		 * cleanly. */
-		statusq_abort();
+		do_prompt_abort();
 	    } else
 #endif /* !DISABLE_BROWSER */
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    if (i == TOGGLE_DOS_KEY) {
 		openfile->fmt = (openfile->fmt == DOS_FILE) ? NIX_FILE :
 			DOS_FILE;
@@ -1678,7 +1797,7 @@ int do_writeout(bool exiting)
 		TOGGLE(BACKUP_FILE);
 		continue;
 	    } else
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
 	    if (i == NANO_PREPEND_KEY) {
 		append = (append == PREPEND) ? OVERWRITE : PREPEND;
 		continue;
@@ -1705,7 +1824,8 @@ int do_writeout(bool exiting)
 		struct stat st;
 
 		if (!stat(answer, &st)) {
-		    i = do_yesno(FALSE, _("File exists, OVERWRITE ? "));
+		    i = do_yesno_prompt(FALSE,
+			_("File exists, OVERWRITE ? "));
 		    if (i == 0 || i == -1)
 			continue;
 		/* If we're using restricted mode, we aren't allowed to
@@ -1715,18 +1835,18 @@ int do_writeout(bool exiting)
 		 * bother showing the "Different Name" prompt. */
 		} else if (!ISSET(RESTRICTED) &&
 			openfile->filename[0] != '\0'
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 			&& (exiting || !openfile->mark_set)
 #endif
 			) {
-		    i = do_yesno(FALSE,
+		    i = do_yesno_prompt(FALSE,
 			_("Save file under DIFFERENT NAME ? "));
 		    if (i == 0 || i == -1)
 			continue;
 		}
 	    }
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    /* Here's where we allow the selected text to be written to
 	     * a separate file.  If we're using restricted mode, this is
 	     * disabled since it allows reading from or writing to files
@@ -1734,7 +1854,7 @@ int do_writeout(bool exiting)
 	    if (!ISSET(RESTRICTED) && !exiting && openfile->mark_set)
 		retval = write_marked_file(answer, NULL, FALSE, append);
 	    else
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
 		retval = write_file(answer, NULL, FALSE, append, FALSE);
 
 	    break;
@@ -1746,6 +1866,8 @@ int do_writeout(bool exiting)
     return retval;
 }
 
+/* Write the current file to disk.  If the mark is on, write the current
+ * marked selection to disk. */
 void do_writeout_void(void)
 {
     do_writeout(FALSE);
@@ -1843,9 +1965,10 @@ int is_dir(const char *buf)
     return ret;
 }
 
-/* These functions (username_tab_completion(), cwd_tab_completion(), and
- * input_tab()) were taken from busybox 0.46 (cmdedit.c).  Here is the
- * notice from that file:
+/* These functions, username_tab_completion(), cwd_tab_completion()
+ * (originally exe_n_cwd_tab_completion()), and input_tab(), were
+ * adapted from busybox 0.46 (cmdedit.c).  Here is the notice from that
+ * file:
  *
  * Termios command line History and Editting, originally
  * intended for NetBSD sh (ash)
@@ -1897,8 +2020,8 @@ char **username_tab_completion(const char *buf, size_t *num_matches,
     return matches;
 }
 
-/* This was originally called exe_n_cwd_tab_completion(), but we're not
- * worried about executables, only filenames :> */
+/* We consider the first buflen characters of buf for filename tab
+ * completion. */
 char **cwd_tab_completion(const char *buf, size_t *num_matches, size_t
 	buflen)
 {
@@ -2080,8 +2203,8 @@ char *input_tab(char *buf, size_t *place, bool *lastwastab, bool *list)
 
 	    *lastwastab = FALSE;
 	    buf = charealloc(buf, common_len + buf_len - *place + 1);
-	    charmove(buf + common_len, buf + *place,
-		buf_len - *place + 1);
+	    charmove(buf + common_len, buf + *place, buf_len -
+		*place + 1);
 	    strncpy(buf, mzero, common_len);
 	    *place = common_len;
 	} else if (*lastwastab == FALSE || num_matches < 2)
@@ -2179,515 +2302,7 @@ const char *tail(const char *foo)
     return tmp;
 }
 
-#ifndef DISABLE_BROWSER
-/* Strip one directory from the end of path. */
-void striponedir(char *path)
-{
-    char *tmp;
-
-    assert(path != NULL);
-
-    tmp = strrchr(path, '/');
-
-    if (tmp != NULL)
- 	*tmp = '\0';
-}
-
-/* Return a list of files contained in the directory path.  *longest is
- * the maximum display length of a file, up to COLS - 1 (but at least
- * 7).  *numents is the number of files.  We assume path exists and is a
- * directory.  If neither is true, we return NULL. */
-char **browser_init(const char *path, int *longest, size_t *numents, DIR
-	*dir)
-{
-    const struct dirent *nextdir;
-    char **filelist;
-    size_t i = 0, path_len;
-
-    assert(dir != NULL);
-
-    *longest = 0;
-
-    while ((nextdir = readdir(dir)) != NULL) {
-	size_t dlen;
-
-	/* Don't show the "." entry. */
-	if (strcmp(nextdir->d_name, ".") == 0)
-	   continue;
-	i++;
-
-	dlen = strlenpt(nextdir->d_name);
-	if (dlen > *longest)
-	    *longest = (dlen > COLS - 1) ? COLS - 1 : dlen;
-    }
-
-    *numents = i;
-    rewinddir(dir);
-    *longest += 10;
-
-    filelist = (char **)nmalloc(*numents * sizeof(char *));
-
-    path_len = strlen(path);
-
-    i = 0;
-
-    while ((nextdir = readdir(dir)) != NULL && i < *numents) {
-	/* Don't show the "." entry. */
-	if (strcmp(nextdir->d_name, ".") == 0)
-	   continue;
-
-	filelist[i] = charalloc(path_len + strlen(nextdir->d_name) + 1);
-	sprintf(filelist[i], "%s%s", path, nextdir->d_name);
-	i++;
-    }
-
-    /* Maybe the number of files in the directory changed between the
-     * first time we scanned and the second.  i is the actual length of
-     * filelist, so record it. */
-    *numents = i;
-    closedir(dir);
-
-    if (*longest > COLS - 1)
-	*longest = COLS - 1;
-    if (*longest < 7)
-	*longest = 7;
-
-    return filelist;
-}
-
-/* Our browser function.  path is the path to start browsing from.
- * Assume path has already been tilde-expanded. */
-char *do_browser(char *path, DIR *dir)
-{
-    int kbinput, longest, selected, width;
-    bool meta_key, func_key, old_const_update = ISSET(CONST_UPDATE);
-    size_t numents;
-    char **filelist, *retval = NULL;
-
-    curs_set(0);
-    blank_statusbar();
-    bottombars(browser_list);
-    wnoutrefresh(bottomwin);
-
-#if !defined(DISABLE_HELP) || !defined(DISABLE_MOUSE)
-    /* Set currshortcut so the user can click in the shortcut area, and
-     * so the browser help screen will come up. */
-    currshortcut = browser_list;
-#endif
-
-    UNSET(CONST_UPDATE);
-
-  change_browser_directory:
-	/* We go here after the user selects a new directory. */
-
-    kbinput = ERR;
-    selected = 0;
-    width = 0;
-
-    path = mallocstrassn(path, get_full_path(path));
-
-    /* Assume that path exists and ends with a slash. */
-    assert(path != NULL && path[strlen(path) - 1] == '/');
-
-    /* Get the list of files. */
-    filelist = browser_init(path, &longest, &numents, dir);
-
-    assert(filelist != NULL);
-
-    /* Sort the list. */
-    qsort(filelist, numents, sizeof(char *), diralphasort);
-
-    titlebar(path);
-
-    /* Loop invariant: Microsoft sucks. */
-    do {
-	bool abort = FALSE;
-	int j, col = 0, editline = 0, fileline;
-	int filecols = 0;
-	    /* Used only if width == 0, to calculate the number of files
-	     * per row below. */
-	struct stat st;
-	char *new_path;
-	    /* Used by the "Go To Directory" prompt. */
-#ifndef DISABLE_MOUSE
-	MEVENT mevent;
-#endif
-
-	check_statusblank();
-
-	/* Compute the line number we're on now, so that we don't divide
-	 * by zero later. */
-	fileline = selected;
-	if (width != 0)
-	    fileline /= width;
-
-	switch (kbinput) {
-#ifndef DISABLE_MOUSE
-	    case KEY_MOUSE:
-		if (getmouse(&mevent) == ERR)
-		    break;
-
-		/* If we clicked in the edit window, we probably clicked
-		 * on a file. */
-		if (wenclose(edit, mevent.y, mevent.x)) {
-		    int old_selected = selected;
-
-		    /* Subtract out the size of topwin. */
-		    mevent.y -= 2 - no_more_space();
-
-		    /* longest is the width of each column.  There are
-		     * two spaces between each column. */
-		    selected = (fileline / editwinrows) * editwinrows *
-			width + mevent.y * width + mevent.x /
-			(longest + 2);
-
-		    /* If they clicked beyond the end of a row, select
-		     * the end of that row. */
-		    if (mevent.x > width * (longest + 2))
-			selected--;
-
-		    /* If we're off the screen, reset to the last item.
-		     * If we clicked the same place as last time, select
-		     * this name! */
-		    if (selected > numents - 1)
-			selected = numents - 1;
-		    else if (old_selected == selected)
-			/* Put back the 'select' key. */
-			unget_kbinput('s', FALSE, FALSE);
-		} else {
-		    /* We must have clicked a shortcut.  Put back the
-		     * equivalent shortcut key. */
-		    int mouse_x, mouse_y;
-		    get_mouseinput(&mouse_x, &mouse_y, TRUE);
-		}
-
-		break;
-#endif
-	    case NANO_PREVLINE_KEY:
-		if (selected >= width)
-		    selected -= width;
-		break;
-	    case NANO_BACK_KEY:
-		if (selected > 0)
-		    selected--;
-		break;
-	    case NANO_NEXTLINE_KEY:
-		if (selected + width <= numents - 1)
-		    selected += width;
-		break;
-	    case NANO_FORWARD_KEY:
-		if (selected < numents - 1)
-		    selected++;
-		break;
-	    case NANO_PREVPAGE_KEY:
-	    case NANO_PREVPAGE_FKEY:
-	    case '-': /* Pico compatibility. */
-		if (selected >= (editwinrows + fileline % editwinrows) *
-			width)
-		    selected -= (editwinrows + fileline % editwinrows) *
-			width;
-		else
-		    selected = 0;
-		break;
-	    case NANO_NEXTPAGE_KEY:
-	    case NANO_NEXTPAGE_FKEY:
-	    case ' ': /* Pico compatibility. */
-		selected += (editwinrows - fileline % editwinrows) *
-			width;
-		if (selected >= numents)
-		    selected = numents - 1;
-		break;
-	    case NANO_HELP_KEY:
-	    case NANO_HELP_FKEY:
-	    case '?': /* Pico compatibility. */
-#ifndef DISABLE_HELP
-		do_help();
-		curs_set(0);
-#else
-		nano_disabled_msg();
-#endif
-		break;
-	    case NANO_ENTER_KEY:
-	    case 'S': /* Pico compatibility. */
-	    case 's':
-		/* You can't move up from "/". */
-		if (strcmp(filelist[selected], "/..") == 0) {
-		    statusbar(_("Can't move up a directory"));
-		    beep();
-		    break;
-		}
-
-#ifndef DISABLE_OPERATINGDIR
-		/* Note: the selected file can be outside the operating
-		 * directory if it's ".." or if it's a symlink to a
-		 * directory outside the operating directory. */
-		if (check_operating_dir(filelist[selected], FALSE)) {
-		    statusbar(
-			_("Can't go outside of %s in restricted mode"),
-			operating_dir);
-		    beep();
-		    break;
-		}
-#endif
-
-		if (stat(filelist[selected], &st) == -1) {
-		    statusbar(_("Error reading %s: %s"),
-			filelist[selected], strerror(errno));
-		    beep();
-		    break;
-		}
-
-		if (!S_ISDIR(st.st_mode)) {
-		    retval = mallocstrcpy(retval, filelist[selected]);
-		    abort = TRUE;
-		    break;
-		}
-
-		dir = opendir(filelist[selected]);
-		if (dir == NULL) {
-		    /* We can't open this dir for some reason.
-		     * Complain. */
-		    statusbar(_("Error reading %s: %s"),
-			filelist[selected], strerror(errno));
-		    break;
-		}
-
-		path = mallocstrcpy(path, filelist[selected]);
-
-		/* Start over again with the new path value. */
-		free_chararray(filelist, numents);
-		goto change_browser_directory;
-
-	    /* Refresh the screen. */
-	    case NANO_REFRESH_KEY:
-		total_redraw();
-		break;
-
-	    /* Go to a specific directory. */
-	    case NANO_GOTOLINE_KEY:
-	    case NANO_GOTOLINE_FKEY:
-	    case 'G': /* Pico compatibility. */
-	    case 'g':
-		curs_set(1);
-
-		j = statusq(FALSE, gotodir_list, "",
-#ifndef NANO_SMALL
-			NULL,
-#endif
-			_("Go To Directory"));
-
-		curs_set(0);
-		bottombars(browser_list);
-
-		if (j < 0) {
-		    statusbar(_("Cancelled"));
-		    break;
-		}
-
-		new_path = real_dir_from_tilde(answer);
-
-		if (new_path[0] != '/') {
-		    new_path = charealloc(new_path, strlen(path) +
-			strlen(answer) + 1);
-		    sprintf(new_path, "%s%s", path, answer);
-		}
-
-#ifndef DISABLE_OPERATINGDIR
-		if (check_operating_dir(new_path, FALSE)) {
-		    statusbar(
-			_("Can't go outside of %s in restricted mode"),
-			operating_dir);
-		    free(new_path);
-		    break;
-		}
-#endif
-
-		dir = opendir(new_path);
-		if (dir == NULL) {
-		    /* We can't open this dir for some reason.
-		     * Complain. */
-		    statusbar(_("Error reading %s: %s"), answer,
-			strerror(errno));
-		    free(new_path);
-		    break;
-		}
-
-		/* Start over again with the new path value. */
-		free(path);
-		path = new_path;
-		free_chararray(filelist, numents);
-		goto change_browser_directory;
-
-	    /* Abort the browser. */
-	    case NANO_EXIT_KEY:
-	    case NANO_EXIT_FKEY:
-	    case 'E': /* Pico compatibility. */
-	    case 'e':
-		abort = TRUE;
-		break;
-	}
-
-	if (abort)
-	    break;
-
-	blank_edit();
-
-	if (width != 0)
-	    j = width * editwinrows *
-		((selected / width) / editwinrows);
-	else
-	    j = 0;
-
-	wmove(edit, 0, 0);
-
-	{
-	    size_t foo_len = mb_cur_max() * 7;
-	    char *foo = charalloc(foo_len + 1);
-
-	    for (; j < numents && editline <= editwinrows - 1; j++) {
-		char *disp = display_string(tail(filelist[j]), 0,
-			longest, FALSE);
-
-		/* Highlight the currently selected file/dir. */
-		if (j == selected)
-		    wattron(edit, A_REVERSE);
-
-		blank_line(edit, editline, col, longest);
-		mvwaddstr(edit, editline, col, disp);
-		free(disp);
-
-		col += longest;
-		filecols++;
-
-		/* Show file info also.  We don't want to report file
-		 * sizes for links, so we use lstat().  Also, stat() and
-		 * lstat() return an error if, for example, the file is
-		 * deleted while the file browser is open.  In that
-		 * case, we report "--" as the file info. */
-		if (lstat(filelist[j], &st) == -1 ||
-			S_ISLNK(st.st_mode)) {
-		    /* Aha!  It's a symlink!  Now, is it a dir?  If so,
-		     * mark it as such. */
-		    if (stat(filelist[j], &st) == 0 &&
-			S_ISDIR(st.st_mode)) {
-			strncpy(foo, _("(dir)"), foo_len);
-			foo[foo_len] = '\0';
-		    } else
-			strcpy(foo, "--");
-		} else if (S_ISDIR(st.st_mode)) {
-		    strncpy(foo, _("(dir)"), foo_len);
-		    foo[foo_len] = '\0';
-		} else if (st.st_size < (1 << 10)) /* less than 1 k. */
-		    sprintf(foo, "%4u  B", (unsigned int)st.st_size);
-		else if (st.st_size < (1 << 20)) /* less than 1 meg. */
-		    sprintf(foo, "%4u KB",
-			(unsigned int)(st.st_size >> 10));
-		else if (st.st_size < (1 << 30)) /* less than 1 gig. */
-		    sprintf(foo, "%4u MB",
-			(unsigned int)(st.st_size >> 20));
-		else
-		    sprintf(foo, "%4u GB",
-			(unsigned int)(st.st_size >> 30));
-
-		mvwaddnstr(edit, editline, col - strlen(foo), foo,
-			foo_len);
-
-		if (j == selected)
-		    wattroff(edit, A_REVERSE);
-
-		/* Add some space between the columns. */
-		col += 2;
-
-		/* If the next entry isn't going to fit on the line,
-		 * move to the next line. */
-		if (col > COLS - longest) {
-		    editline++;
-		    col = 0;
-		    if (width == 0)
-			width = filecols;
-		}
-
-		wmove(edit, editline, col);
-	    }
-
-	    free(foo);
-	}
-
-	wnoutrefresh(edit);
-    } while ((kbinput = get_kbinput(edit, &meta_key, &func_key)) !=
-	NANO_EXIT_KEY && kbinput != NANO_EXIT_FKEY);
-
-    blank_edit();
-    titlebar(NULL);
-    edit_refresh();
-    curs_set(1);
-    if (old_const_update)
-	SET(CONST_UPDATE);
-
-    /* Clean up. */
-    free_chararray(filelist, numents);
-    free(path);
-
-    return retval;
-}
-
-/* The file browser front end.  We check to see if inpath has a dir in
- * it.  If it does, we start do_browser() from there.  Otherwise, we
- * start do_browser() from the current directory. */
-char *do_browse_from(const char *inpath)
-{
-    struct stat st;
-    char *path;
-	/* This holds the tilde-expanded version of inpath. */
-    DIR *dir = NULL;
-
-    assert(inpath != NULL);
-
-    path = real_dir_from_tilde(inpath);
-
-    /* Perhaps path is a directory.  If so, we'll pass it to
-     * do_browser().  Or perhaps path is a directory / a file.  If so,
-     * we'll try stripping off the last path element and passing it to
-     * do_browser().  Or perhaps path doesn't have a directory portion
-     * at all.  If so, we'll just pass the current directory to
-     * do_browser(). */
-    if (stat(path, &st) == -1 || !S_ISDIR(st.st_mode)) {
-	striponedir(path);
-	if (stat(path, &st) == -1 || !S_ISDIR(st.st_mode)) {
-	    free(path);
-
-	    path = charalloc(PATH_MAX + 1);
-	    path = getcwd(path, PATH_MAX + 1);
-
-	    if (path != NULL)
-		align(&path);
-	}
-    }
-
-#ifndef DISABLE_OPERATINGDIR
-    /* If the resulting path isn't in the operating directory, use
-     * the operating directory instead. */
-    if (check_operating_dir(path, FALSE)) {
-	if (path != NULL)
-	    free(path);
-	path = mallocstrcpy(NULL, operating_dir);
-    }
-#endif
-
-    if (path != NULL)
-	dir = opendir(path);
-
-    if (dir == NULL) {
-	beep();
-	free(path);
-	return NULL;
-    }
-
-    return do_browser(path, dir);
-}
-#endif /* !DISABLE_BROWSER */
-
-#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+#if !defined(NANO_TINY) && defined(ENABLE_NANORC)
 /* Return $HOME/.nano_history, or NULL if we can't find the homedir.
  * The string is dynamically allocated, and should be freed. */
 char *histfilename(void)
@@ -2720,7 +2335,7 @@ void load_history(void)
 		rcfile_error(N_("Error reading %s: %s"), nanohist,
 			strerror(errno));
 		fprintf(stderr,
-			_("\nPress Return to continue starting nano\n"));
+			_("\nPress Enter to continue starting nano\n"));
 		while (getchar() != '\n')
 		    ;
 	    }
@@ -2752,6 +2367,9 @@ void load_history(void)
     }
 }
 
+/* Write the lines of a history list, starting with the line at h, to
+ * the open file at hist.  Return TRUE if the write succeeded, and FALSE
+ * otherwise. */
 bool writehist(FILE *hist, filestruct *h)
 {
     filestruct *p;
@@ -2805,4 +2423,4 @@ void save_history(void)
 	free(nanohist);
     }
 }
-#endif /* !NANO_SMALL && ENABLE_NANORC */
+#endif /* !NANO_TINY && ENABLE_NANORC */
