@@ -1,8 +1,9 @@
-/* $Id: move.c,v 1.47 2005/09/13 04:53:44 dolorous Exp $ */
+/* $Id: move.c,v 1.58 2005/12/08 07:09:08 dolorous Exp $ */
 /**************************************************************************
  *   move.c                                                               *
  *                                                                        *
- *   Copyright (C) 1999-2005 Chris Allegretta                             *
+ *   Copyright (C) 1999-2004 Chris Allegretta                             *
+ *   Copyright (C) 2005 David Lawrence Ramsey                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -20,14 +21,12 @@
  *                                                                        *
  **************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "proto.h"
 
 #include <string.h>
 #include <ctype.h>
-#include "proto.h"
 
+/* Move to the first line of the file. */
 void do_first_line(void)
 {
     const filestruct *current_save = openfile->current;
@@ -42,6 +41,7 @@ void do_first_line(void)
     edit_redraw(current_save, pww_save);
 }
 
+/* Move to the last line of the file. */
 void do_last_line(void)
 {
     const filestruct *current_save = openfile->current;
@@ -50,13 +50,14 @@ void do_last_line(void)
     check_statusblank();
 
     openfile->current = openfile->filebot;
-    openfile->current_x = 0;
-    openfile->placewewant = 0;
+    openfile->current_x = strlen(openfile->filebot->data);
+    openfile->placewewant = xplustabs();
     openfile->current_y = editwinrows - 1;
 
     edit_redraw(current_save, pww_save);
 }
 
+/* Move up one page. */
 void do_page_up(void)
 {
     int i;
@@ -77,17 +78,17 @@ void do_page_up(void)
 
     /* If we're not in smooth scrolling mode, put the cursor at the
      * beginning of the top line of the edit window, as Pico does. */
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     if (!ISSET(SMOOTH_SCROLL)) {
 #endif
 	openfile->current = openfile->edittop;
 	openfile->placewewant = 0;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     }
 #endif
 
-    for (i = editwinrows - 2; i > 0 && openfile->current->prev != NULL;
-	i--)
+    for (i = editwinrows - 2; i > 0 && openfile->current !=
+	openfile->fileage; i--)
 	openfile->current = openfile->current->prev;
 
     openfile->current_x = actual_x(openfile->current->data,
@@ -97,6 +98,7 @@ void do_page_up(void)
     edit_scroll(UP, editwinrows - 2);
 }
 
+/* Move down one page. */
 void do_page_down(void)
 {
     int i;
@@ -118,17 +120,17 @@ void do_page_down(void)
 
     /* If we're not in smooth scrolling mode, put the cursor at the
      * beginning of the top line of the edit window, as Pico does. */
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     if (!ISSET(SMOOTH_SCROLL)) {
 #endif
 	openfile->current = openfile->edittop;
 	openfile->placewewant = 0;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     }
 #endif
 
-    for (i = editwinrows - 2; i > 0 && openfile->current->next != NULL;
-	i--)
+    for (i = editwinrows - 2; i > 0 && openfile->current !=
+	openfile->filebot; i--)
 	openfile->current = openfile->current->next;
 
     openfile->current_x = actual_x(openfile->current->data,
@@ -139,8 +141,9 @@ void do_page_down(void)
 }
 
 #ifndef DISABLE_JUSTIFY
-/* Move up to the last beginning-of-paragraph line before the current
- * line. */
+/* Move up to the beginning of the last beginning-of-paragraph line
+ * before the current line.  If allow_update is TRUE, update the screen
+ * afterwards. */
 void do_para_begin(bool allow_update)
 {
     const filestruct *current_save = openfile->current;
@@ -148,29 +151,33 @@ void do_para_begin(bool allow_update)
 
     check_statusblank();
 
-    openfile->current_x = 0;
-    openfile->placewewant = 0;
-
-    if (openfile->current->prev != NULL) {
+    if (openfile->current != openfile->fileage) {
 	do {
 	    openfile->current = openfile->current->prev;
 	    openfile->current_y--;
 	} while (!begpar(openfile->current));
     }
 
+    openfile->current_x = 0;
+    openfile->placewewant = 0;
+
     if (allow_update)
 	edit_redraw(current_save, pww_save);
 }
 
+/* Move up to the beginning of the last beginning-of-paragraph line
+ * before the current line, and update the screen afterwards. */
 void do_para_begin_void(void)
 {
     do_para_begin(TRUE);
 }
 
-/* Move down to the end of a paragraph, then one line farther.  A line
- * is the last line of a paragraph if it is in a paragraph, and the next
- * line either is a beginning-of-paragraph line or isn't in a
- * paragraph. */
+/* Move down to the beginning of the last line of the current paragraph.
+ * Then move down one line farther if there is such a line, or to the
+ * end of the current line if not.  If allow_update is TRUE, update the
+ * screen afterwards.  A line is the last line of a paragraph if it is
+ * in a paragraph, and the next line either is the beginning line of a
+ * paragraph or isn't in a paragraph. */
 void do_para_end(bool allow_update)
 {
     const filestruct *const current_save = openfile->current;
@@ -178,37 +185,44 @@ void do_para_end(bool allow_update)
 
     check_statusblank();
 
-    openfile->current_x = 0;
-    openfile->placewewant = 0;
-
-    while (openfile->current->next != NULL && !inpar(openfile->current))
+    while (openfile->current != openfile->filebot &&
+	!inpar(openfile->current))
 	openfile->current = openfile->current->next;
 
-    while (openfile->current->next != NULL &&
+    while (openfile->current != openfile->filebot &&
 	inpar(openfile->current->next) &&
 	!begpar(openfile->current->next)) {
 	openfile->current = openfile->current->next;
 	openfile->current_y++;
     }
 
-    if (openfile->current->next != NULL)
+    if (openfile->current != openfile->filebot) {
 	openfile->current = openfile->current->next;
+	openfile->current_x = 0;
+	openfile->placewewant = 0;
+    } else {
+	openfile->current_x = strlen(openfile->current->data);
+	openfile->placewewant = xplustabs();
+    }
 
     if (allow_update)
 	edit_redraw(current_save, pww_save);
 }
 
+/* Move down to the beginning of the last line of the current paragraph.
+ * Then move down one line farther if there is such a line, or to the
+ * end of the current line if not, and update the screen afterwards. */
 void do_para_end_void(void)
 {
     do_para_end(TRUE);
 }
 #endif /* !DISABLE_JUSTIFY */
 
-#ifndef NANO_SMALL
-/* Move to the next word in the current filestruct.  If allow_punct is
- * TRUE, treat punctuation as part of a word.  If allow_update is TRUE,
- * update the screen afterward.  Return TRUE if we started on a word,
- * and FALSE otherwise. */
+#ifndef NANO_TINY
+/* Move to the next word in the file.  If allow_punct is TRUE, treat
+ * punctuation as part of a word.  If allow_update is TRUE, update the
+ * screen afterwards.  Return TRUE if we started on a word, and FALSE
+ * otherwise. */
 bool do_next_word(bool allow_punct, bool allow_update)
 {
     size_t pww_save = openfile->placewewant;
@@ -272,7 +286,7 @@ bool do_next_word(bool allow_punct, bool allow_update)
 	if (!end_line)
 	    break;
 
-	if (openfile->current->next != NULL) {
+	if (openfile->current != openfile->filebot) {
 	    end_line = FALSE;
 	    openfile->current_x = 0;
 	}
@@ -295,15 +309,18 @@ bool do_next_word(bool allow_punct, bool allow_update)
     return started_on_word;
 }
 
+/* Move to the next word in the file, treating punctuation as part of a
+ * word if the WORD_BOUNDS flag is set, and update the screen
+ * afterwards. */
 void do_next_word_void(void)
 {
     do_next_word(ISSET(WORD_BOUNDS), TRUE);
 }
 
-/* Move to the previous word in the current filestruct.  If allow_punct
- * is TRUE, treat punctuation as part of a word.  If allow_update is
- * TRUE, update the screen afterward.  Return TRUE if we started on a
- * word, and FALSE otherwise. */
+/* Move to the previous word in the file.  If allow_punct is TRUE, treat
+ * punctuation as part of a word.  If allow_update is TRUE, update the
+ * screen afterwards.  Return TRUE if we started on a word, and FALSE
+ * otherwise. */
 bool do_prev_word(bool allow_punct, bool allow_update)
 {
     size_t pww_save = openfile->placewewant;
@@ -372,7 +389,7 @@ bool do_prev_word(bool allow_punct, bool allow_update)
 	if (!begin_line)
 	    break;
 
-	if (openfile->current->prev != NULL) {
+	if (openfile->current != openfile->fileage) {
 	    begin_line = FALSE;
 	    openfile->current_x = strlen(openfile->current->prev->data);
 	}
@@ -426,19 +443,26 @@ bool do_prev_word(bool allow_punct, bool allow_update)
     return started_on_word;
 }
 
+/* Move to the previous word in the file, treating punctuation as part
+ * of a word if the WORD_BOUNDS flag is set, and update the screen
+ * afterwards. */
 void do_prev_word_void(void)
 {
     do_prev_word(ISSET(WORD_BOUNDS), TRUE);
 }
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
 
+/* Move to the beginning of the current line.  If the SMART_HOME flag is
+ * set, move to the first non-whitespace character of the current line
+ * if we're not already there, or to the beginning of the current line
+ * if we are. */
 void do_home(void)
 {
     size_t pww_save = openfile->placewewant;
 
     check_statusblank();
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     if (ISSET(SMART_HOME)) {
 	size_t current_x_save = openfile->current_x;
 
@@ -453,7 +477,7 @@ void do_home(void)
 #endif
 	openfile->current_x = 0;
 	openfile->placewewant = 0;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     }
 #endif
 
@@ -461,6 +485,7 @@ void do_home(void)
 	update_line(openfile->current, openfile->current_x);
 }
 
+/* Move to the end of the current line. */
 void do_end(void)
 {
     size_t pww_save = openfile->placewewant;
@@ -474,6 +499,7 @@ void do_end(void)
 	update_line(openfile->current, openfile->current_x);
 }
 
+/* Move up one line. */
 void do_up(void)
 {
     check_statusblank();
@@ -483,7 +509,7 @@ void do_up(void)
 #endif
 
     /* If we're at the top of the file, get out. */
-    if (openfile->current->prev == NULL)
+    if (openfile->current == openfile->fileage)
 	return;
 
     assert(openfile->current_y == openfile->current->lineno - openfile->edittop->lineno);
@@ -498,7 +524,7 @@ void do_up(void)
      * a page if we're not. */
     if (openfile->current_y == 0)
 	edit_scroll(UP,
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 		ISSET(SMOOTH_SCROLL) ? 1 :
 #endif
 		editwinrows / 2);
@@ -512,6 +538,33 @@ void do_up(void)
     }
 }
 
+#ifndef NANO_TINY
+/* Scroll up one line without scrolling the cursor. */
+void do_scroll_up(void)
+{
+    check_statusblank();
+
+#ifndef DISABLE_WRAPPING
+    wrap_reset();
+#endif
+
+    /* If the top of the file is onscreen, get out. */
+    if (openfile->edittop == openfile->fileage)
+	return;
+
+    assert(openfile->current_y == openfile->current->lineno - openfile->edittop->lineno);
+
+    /* Move the current line of the edit window up. */
+    openfile->current = openfile->current->prev;
+    openfile->current_x = actual_x(openfile->current->data,
+	openfile->placewewant);
+
+    /* Scroll the edit window up one line. */
+    edit_scroll(UP, 1);
+}
+#endif /* !NANO_TINY */
+
+/* Move down one line. */
 void do_down(void)
 {
     check_statusblank();
@@ -521,7 +574,7 @@ void do_down(void)
 #endif
 
     /* If we're at the bottom of the file, get out. */
-    if (openfile->current->next == NULL)
+    if (openfile->current == openfile->filebot)
 	return;
 
     assert(openfile->current_y == openfile->current->lineno - openfile->edittop->lineno);
@@ -536,7 +589,7 @@ void do_down(void)
      * half a page if we're not. */
     if (openfile->current_y == editwinrows - 1)
 	edit_scroll(DOWN,
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 		ISSET(SMOOTH_SCROLL) ? 1 :
 #endif
 		editwinrows / 2);
@@ -550,6 +603,33 @@ void do_down(void)
     }
 }
 
+#ifndef NANO_TINY
+/* Scroll down one line without scrolling the cursor. */
+void do_scroll_down(void)
+{
+    check_statusblank();
+
+#ifndef DISABLE_WRAPPING
+    wrap_reset();
+#endif
+
+    /* If we're at the bottom of the file, get out. */
+    if (openfile->current == openfile->filebot)
+	return;
+
+    assert(openfile->current_y == openfile->current->lineno - openfile->edittop->lineno);
+
+    /* Move the current line of the edit window down. */
+    openfile->current = openfile->current->next;
+    openfile->current_x = actual_x(openfile->current->data,
+	openfile->placewewant);
+
+    /* Scroll the edit window down one line. */
+    edit_scroll(DOWN, 1);
+}
+#endif /* !NANO_TINY */
+
+/* Move left one character. */
 void do_left(void)
 {
     size_t pww_save = openfile->placewewant;
@@ -570,6 +650,7 @@ void do_left(void)
 	update_line(openfile->current, openfile->current_x);
 }
 
+/* Move right one character. */
 void do_right(void)
 {
     size_t pww_save = openfile->placewewant;
@@ -581,7 +662,7 @@ void do_right(void)
     if (openfile->current->data[openfile->current_x] != '\0')
 	openfile->current_x = move_mbright(openfile->current->data,
 		openfile->current_x);
-    else if (openfile->current->next != NULL) {
+    else if (openfile->current != openfile->filebot) {
 	do_down();
 	openfile->current_x = 0;
     }
