@@ -1,8 +1,9 @@
-/* $Id: nano.c,v 1.493 2005/09/02 04:42:44 dolorous Exp $ */
+/* $Id: nano.c,v 1.506 2005/12/08 07:09:08 dolorous Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
- *   Copyright (C) 1999-2005 Chris Allegretta                             *
+ *   Copyright (C) 1999-2004 Chris Allegretta                             *
+ *   Copyright (C) 2005 David Lawrence Ramsey                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -20,9 +21,7 @@
  *                                                                        *
  **************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "proto.h"
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -35,26 +34,27 @@
 #include <errno.h>
 #include <ctype.h>
 #include <locale.h>
-#include "proto.h"
-
-#ifdef HAVE_TERMIOS_H
-#include <termios.h>
-#endif
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
-#ifndef NANO_SMALL
+#ifdef HAVE_TERMIOS_H
+#include <termios.h>
+#endif
+
+#ifndef NANO_TINY
 #include <setjmp.h>
 #endif
 
-static struct termios oldterm;	/* The user's original term settings. */
-static struct sigaction act;	/* For all our fun signal handlers. */
+static struct termios oldterm;
+	/* The user's original terminal settings. */
+static struct sigaction act;
+	/* For all our fun signal handlers. */
 
-#ifndef NANO_SMALL
-static sigjmp_buf jmpbuf;	/* Used to return to main() after a
-				 * SIGWINCH. */
+#ifndef NANO_TINY
+static sigjmp_buf jmpbuf;
+	/* Used to return to main() after a SIGWINCH. */
 #endif
 
 /* Create a new filestruct node.  Note that we specifically do not set
@@ -230,7 +230,7 @@ partition *partition_filestruct(filestruct *top, size_t top_x,
 }
 
 /* Unpartition a filestruct so it begins at (fileage, 0) and ends at
- * (filebot, strlen(filebot)) again. */
+ * (filebot, strlen(filebot->data)) again. */
 void unpartition_filestruct(partition **p)
 {
     char *tmp;
@@ -283,7 +283,7 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
 {
     filestruct *top_save;
     bool edittop_inside;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     bool mark_inside = FALSE;
 #endif
 
@@ -301,7 +301,7 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
     edittop_inside = (openfile->edittop->lineno >=
 	openfile->fileage->lineno && openfile->edittop->lineno <=
 	openfile->filebot->lineno);
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     if (openfile->mark_set)
 	mark_inside = (openfile->mark_begin->lineno >=
 		openfile->fileage->lineno &&
@@ -357,7 +357,7 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
     openfile->fileage = (filestruct *)nmalloc(sizeof(filestruct));
     openfile->fileage->data = mallocstrcpy(NULL, "");
     openfile->filebot = openfile->fileage;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     if (mark_inside) {
 	openfile->mark_begin = openfile->fileage;
 	openfile->mark_begin_x = top_x;
@@ -378,7 +378,7 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
      * it in range of current. */
     if (edittop_inside)
 	edit_update(
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 		ISSET(SMOOTH_SCROLL) ? NONE :
 #endif
 		CENTER);
@@ -387,8 +387,9 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
      * partition. */
     renumber(top_save);
 
-    /* If the text doesn't end with a magicline, add a new magicline. */
-    if (openfile->filebot->data[0] != '\0')
+    /* If the NO_NEWLINES flag isn't set, and the text doesn't end with
+     * a magicline, add a new magicline. */
+    if (!ISSET(NO_NEWLINES) && openfile->filebot->data[0] != '\0')
 	new_magicline();
 }
 
@@ -447,8 +448,9 @@ void copy_from_filestruct(filestruct *file_top, filestruct *file_bot)
      * partition. */
     renumber(top_save);
 
-    /* If the text doesn't end with a magicline, add a new magicline. */
-    if (openfile->filebot->data[0] != '\0')
+    /* If the NO_NEWLINES flag isn't set, and the text doesn't end with
+     * a magicline, add a new magicline. */
+    if (!ISSET(NO_NEWLINES) && openfile->filebot->data[0] != '\0')
 	new_magicline();
 }
 
@@ -499,7 +501,7 @@ void delete_opennode(openfilestruct *fileptr)
 
     free(fileptr->filename);
     free_filestruct(fileptr->fileage);
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     if (fileptr->current_stat != NULL)
 	free(fileptr->current_stat);
 #endif
@@ -523,6 +525,7 @@ void free_openfilestruct(openfilestruct *src)
 }
 #endif
 
+/* Display a warning about a key disabled in view mode. */
 void print_view_warning(void)
 {
     statusbar(_("Key illegal in VIEW mode"));
@@ -542,7 +545,7 @@ void finish(void)
     /* Restore the old terminal settings. */
     tcsetattr(0, TCSANOW, &oldterm);
 
-#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+#if !defined(NANO_TINY) && defined(ENABLE_NANORC)
     if (!ISSET(NO_RCFILE) && ISSET(HISTORYLOG))
 	save_history();
 #endif
@@ -597,6 +600,8 @@ void die(const char *msg, ...)
     exit(1);
 }
 
+/* Save the current file under the name spacified in die_filename, which
+ * is modified to be unique if necessary. */
 void die_save_file(const char *die_filename)
 {
     char *retval;
@@ -630,11 +635,12 @@ void die_save_file(const char *die_filename)
     free(retval);
 }
 
+/* Initialize the three window portions nano uses. */
 void window_init(void)
 {
     /* If the screen height is too small, get out. */
     editwinrows = LINES - 5 + no_more_space() + no_help();
-    if (editwinrows < MIN_EDITOR_ROWS)
+    if (COLS < MIN_EDITOR_COLS || editwinrows < MIN_EDITOR_ROWS)
 	die(_("Window size is too small for nano...\n"));
 
 #ifndef DISABLE_WRAPJUSTIFY
@@ -668,6 +674,7 @@ void window_init(void)
 }
 
 #ifndef DISABLE_MOUSE
+/* Initialize mouse support. */
 void mouse_init(void)
 {
     if (ISSET(USE_MOUSE)) {
@@ -678,329 +685,6 @@ void mouse_init(void)
 }
 #endif
 
-#ifndef DISABLE_HELP
-/* This function allocates help_text, and stores the help string in it. 
- * help_text should be NULL initially. */
-void help_init(void)
-{
-    size_t allocsize = 0;	/* Space needed for help_text. */
-    const char *htx[3];		/* Untranslated help message.  We break
-				 * it up into three chunks in case the
-				 * full string is too long for the
-				 * compiler to handle. */
-    char *ptr;
-    const shortcut *s;
-#ifndef NANO_SMALL
-    const toggle *t;
-#ifdef ENABLE_NANORC
-    bool old_whitespace = ISSET(WHITESPACE_DISPLAY);
-
-    UNSET(WHITESPACE_DISPLAY);
-#endif
-#endif
-
-    /* First, set up the initial help text for the current function. */
-    if (currshortcut == whereis_list || currshortcut == replace_list ||
-	currshortcut == replace_list_2) {
-	htx[0] = N_("Search Command Help Text\n\n "
-		"Enter the words or characters you would like to "
-		"search for, and then press Enter.  If there is a "
-		"match for the text you entered, the screen will be "
-		"updated to the location of the nearest match for the "
-		"search string.\n\n The previous search string will be "
-		"shown in brackets after the search prompt.  Hitting "
-		"Enter without entering any text will perform the "
-		"previous search.  ");
-	htx[1] = N_("If you have selected text with the mark and then "
-		"search to replace, only matches in the selected text "
-		"will be replaced.\n\n The following function keys are "
-		"available in Search mode:\n\n");
-	htx[2] = NULL;
-    } else if (currshortcut == gotoline_list) {
-	htx[0] = N_("Go To Line Help Text\n\n "
-		"Enter the line number that you wish to go to and hit "
-		"Enter.  If there are fewer lines of text than the "
-		"number you entered, you will be brought to the last "
-		"line of the file.\n\n The following function keys are "
-		"available in Go To Line mode:\n\n");
-	htx[1] = NULL;
-	htx[2] = NULL;
-    } else if (currshortcut == insertfile_list) {
-	htx[0] = N_("Insert File Help Text\n\n "
-		"Type in the name of a file to be inserted into the "
-		"current file buffer at the current cursor "
-		"location.\n\n If you have compiled nano with multiple "
-		"file buffer support, and enable multiple file buffers "
-		"with the -F or --multibuffer command line flags, the "
-		"Meta-F toggle, or a nanorc file, inserting a file "
-		"will cause it to be loaded into a separate buffer "
-		"(use Meta-< and > to switch between file buffers).  ");
-	htx[1] = N_("If you need another blank buffer, do not enter "
-		"any filename, or type in a nonexistent filename at "
-		"the prompt and press Enter.\n\n The following "
-		"function keys are available in Insert File mode:\n\n");
-	htx[2] = NULL;
-    } else if (currshortcut == writefile_list) {
-	htx[0] = N_("Write File Help Text\n\n "
-		"Type the name that you wish to save the current file "
-		"as and press Enter to save the file.\n\n If you have "
-		"selected text with the mark, you will be prompted to "
-		"save only the selected portion to a separate file.  To "
-		"reduce the chance of overwriting the current file with "
-		"just a portion of it, the current filename is not the "
-		"default in this mode.\n\n The following function keys "
-		"are available in Write File mode:\n\n");
-	htx[1] = NULL;
-	htx[2] = NULL;
-    }
-#ifndef DISABLE_BROWSER
-    else if (currshortcut == browser_list) {
-	htx[0] = N_("File Browser Help Text\n\n "
-		"The file browser is used to visually browse the "
-		"directory structure to select a file for reading "
-		"or writing.  You may use the arrow keys or Page Up/"
-		"Down to browse through the files, and S or Enter to "
-		"choose the selected file or enter the selected "
-		"directory.  To move up one level, select the "
-		"directory called \"..\" at the top of the file "
-		"list.\n\n The following function keys are available "
-		"in the file browser:\n\n");
-	htx[1] = NULL;
-	htx[2] = NULL;
-    } else if (currshortcut == gotodir_list) {
-	htx[0] = N_("Browser Go To Directory Help Text\n\n "
-		"Enter the name of the directory you would like to "
-		"browse to.\n\n If tab completion has not been "
-		"disabled, you can use the Tab key to (attempt to) "
-		"automatically complete the directory name.\n\n The "
-		"following function keys are available in Browser Go "
-		"To Directory mode:\n\n");
-	htx[1] = NULL;
-	htx[2] = NULL;
-    }
-#endif
-#ifndef DISABLE_SPELLER
-    else if (currshortcut == spell_list) {
-	htx[0] = N_("Spell Check Help Text\n\n "
-		"The spell checker checks the spelling of all text in "
-		"the current file.  When an unknown word is "
-		"encountered, it is highlighted and a replacement can "
-		"be edited.  It will then prompt to replace every "
-		"instance of the given misspelled word in the current "
-		"file, or, if you have selected text with the mark, in "
-		"the selected text.\n\n The following other functions "
-		"are available in Spell Check mode:\n\n");
-	htx[1] = NULL;
-	htx[2] = NULL;
-    }
-#endif
-#ifndef NANO_SMALL
-    else if (currshortcut == extcmd_list) {
-	htx[0] = N_("Execute Command Help Text\n\n "
-		"This menu allows you to insert the output of a "
-		"command run by the shell into the current buffer (or "
-		"a new buffer in multiple file buffer mode). If you "
-		"need another blank buffer, do not enter any "
-		"command.\n\n The following keys are available in "
-		"Execute Command mode:\n\n");
-	htx[1] = NULL;
-	htx[2] = NULL;
-    }
-#endif
-    else {
-	/* Default to the main help list. */
-	htx[0] = N_(" nano help text\n\n "
-		"The nano editor is designed to emulate the "
-		"functionality and ease-of-use of the UW Pico text "
-		"editor.  There are four main sections of the editor.  "
-		"The top line shows the program version, the current "
-		"filename being edited, and whether or not the file "
-		"has been modified.  Next is the main editor window "
-		"showing the file being edited.  The status line is "
-		"the third line from the bottom and shows important "
-		"messages.  The bottom two lines show the most "
-		"commonly used shortcuts in the editor.\n\n ");
-	htx[1] = N_("The notation for shortcuts is as follows: "
-		"Control-key sequences are notated with a caret (^) "
-		"symbol and can be entered either by using the Control "
-		"(Ctrl) key or pressing the Escape (Esc) key twice.  "
-		"Escape-key sequences are notated with the Meta (M) "
-		"symbol and can be entered using either the Esc, Alt, "
-		"or Meta key depending on your keyboard setup.  ");
-	htx[2] = N_("Also, pressing Esc twice and then typing a "
-		"three-digit decimal number from 000 to 255 will enter "
-		"the character with the corresponding value.  The "
-		"following keystrokes are available in the main editor "
-		"window.  Alternative keys are shown in "
-		"parentheses:\n\n");
-    }
-
-    htx[0] = _(htx[0]);
-    if (htx[1] != NULL)
-	htx[1] = _(htx[1]);
-    if (htx[2] != NULL)
-	htx[2] = _(htx[2]);
-
-    allocsize += strlen(htx[0]);
-    if (htx[1] != NULL)
-	allocsize += strlen(htx[1]);
-    if (htx[2] != NULL)
-	allocsize += strlen(htx[2]);
-
-    /* The space needed for the shortcut lists, at most COLS characters,
-     * plus '\n'. */
-    allocsize += (COLS < 24 ? (24 * mb_cur_max()) :
-	((COLS + 1) * mb_cur_max())) * length_of_list(currshortcut);
-
-#ifndef NANO_SMALL
-    /* If we're on the main list, we also count the toggle help text.
-     * Each line has "M-%c\t\t\t", which fills 24 columns, plus a space,
-     * plus translated text, plus '\n'. */
-    if (currshortcut == main_list) {
-	size_t endis_len = strlen(_("enable/disable"));
-
-	for (t = toggles; t != NULL; t = t->next)
-	    allocsize += 8 + strlen(t->desc) + endis_len;
-    }
-#endif
-
-    /* help_text has been freed and set to NULL unless the user resized
-     * while in the help screen. */
-    free(help_text);
-
-    /* Allocate space for the help text. */
-    help_text = charalloc(allocsize + 1);
-
-    /* Now add the text we want. */
-    strcpy(help_text, htx[0]);
-    if (htx[1] != NULL)
-	strcat(help_text, htx[1]);
-    if (htx[2] != NULL)
-	strcat(help_text, htx[2]);
-
-    ptr = help_text + strlen(help_text);
-
-    /* Now add our shortcut info.  Assume that each shortcut has, at the
-     * very least, an equivalent control key, an equivalent primary meta
-     * key sequence, or both.  Also assume that the meta key values are
-     * not control characters.  We can display a maximum of 3 shortcut
-     * entries. */
-    for (s = currshortcut; s != NULL; s = s->next) {
-	int entries = 0;
-
-	/* Control key. */
-	if (s->ctrlval != NANO_NO_KEY) {
-	    entries++;
-	    /* Yucky sentinel values that we can't handle a better
-	     * way. */
-	    if (s->ctrlval == NANO_CONTROL_SPACE) {
-		char *space_ptr = display_string(_("Space"), 0, 6,
-			FALSE);
-
-		ptr += sprintf(ptr, "^%s", space_ptr);
-
-		free(space_ptr);
-	    } else if (s->ctrlval == NANO_CONTROL_8)
-		ptr += sprintf(ptr, "^?");
-	    /* Normal values. */
-	    else
-		ptr += sprintf(ptr, "^%c", s->ctrlval + 64);
-	    *(ptr++) = '\t';
-	}
-
-	/* Function key. */
-	if (s->funcval != NANO_NO_KEY) {
-	    entries++;
-	    /* If this is the first entry, put it in the middle. */
-	    if (entries == 1) {
-		entries++;
-		*(ptr++) = '\t';
-	    }
-	    ptr += sprintf(ptr, "(F%d)", s->funcval - KEY_F0);
-	    *(ptr++) = '\t';
-	}
-
-	/* Primary meta key sequence.  If it's the first entry, don't
-	 * put parentheses around it. */
-	if (s->metaval != NANO_NO_KEY) {
-	    entries++;
-	    /* If this is the last entry, put it at the end. */
-	    if (entries == 2 && s->miscval == NANO_NO_KEY) {
-		entries++;
-		*(ptr++) = '\t';
-	    }
-	    /* Yucky sentinel values that we can't handle a better
-	     * way. */
-	    if (s->metaval == NANO_ALT_SPACE && entries == 1) {
-		char *space_ptr = display_string(_("Space"), 0, 5,
-			FALSE);
-
-		ptr += sprintf(ptr, "M-%s", space_ptr);
-
-		free(space_ptr);
-	    } else
-		/* Normal values. */
-		ptr += sprintf(ptr, (entries == 1) ? "M-%c" : "(M-%c)",
-			toupper(s->metaval));
-	    *(ptr++) = '\t';
-	}
-
-	/* Miscellaneous meta key sequence. */
-	if (entries < 3 && s->miscval != NANO_NO_KEY) {
-	    entries++;
-	    /* If this is the last entry, put it at the end. */
-	    if (entries == 2) {
-		entries++;
-		*(ptr++) = '\t';
-	    }
-	    ptr += sprintf(ptr, "(M-%c)", toupper(s->miscval));
-	    *(ptr++) = '\t';
-	}
-
-	/* Make sure all the help text starts at the same place. */
-	while (entries < 3) {
-	    entries++;
-	    *(ptr++) = '\t';
-	}
-
-	assert(s->help != NULL);
-
-	if (COLS > 24) {
-	    char *help_ptr = display_string(s->help, 0, COLS - 24,
-		FALSE);
-
-	    ptr += sprintf(ptr, help_ptr);
-
-	    free(help_ptr);
-	}
-
-	ptr += sprintf(ptr, "\n");
-    }
-
-#ifndef NANO_SMALL
-    /* And the toggles... */
-    if (currshortcut == main_list) {
-	for (t = toggles; t != NULL; t = t->next) {
-
-	    assert(t->desc != NULL);
-
-	    ptr += sprintf(ptr, "M-%c\t\t\t%s %s\n", toupper(t->val),
-		t->desc, _("enable/disable"));
-	}
-    }
-
-#ifdef ENABLE_NANORC
-    if (old_whitespace)
-	SET(WHITESPACE_DISPLAY);
-#endif
-#endif
-
-    /* If all went well, we didn't overwrite the allocated space for
-     * help_text. */
-    assert(strlen(help_text) <= allocsize + 1);
-}
-#endif
-
 #ifdef HAVE_GETOPT_LONG
 #define print1opt(shortflag, longflag, desc) print1opt_full(shortflag, longflag, desc)
 #else
@@ -1033,6 +717,7 @@ void print1opt_full(const char *shortflag
     printf("\n");
 }
 
+/* Explain how to properly use nano and its command line options. */
 void usage(void)
 {
 #ifdef HAVE_GETOPT_LONG
@@ -1047,7 +732,7 @@ void usage(void)
     print1opt("-h, -?", "--help", N_("Show this message"));
     print1opt(_("+LINE,COLUMN"), "",
 	N_("Start at line LINE, column COLUMN"));
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     print1opt("-A", "--smarthome", N_("Enable smart home key"));
     print1opt("-B", "--backup", N_("Save backups of existing files"));
     print1opt(_("-C [dir]"), _("--backupdir=[dir]"),
@@ -1059,7 +744,7 @@ void usage(void)
     print1opt("-F", "--multibuffer", N_("Enable multiple file buffers"));
 #endif
 #ifdef ENABLE_NANORC
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     print1opt("-H", "--historylog",
 	N_("Log & read search/replace string history"));
 #endif
@@ -1068,7 +753,9 @@ void usage(void)
 #endif
     print1opt("-K", "--rebindkeypad",
 	N_("Fix numeric keypad key confusion problem"));
-#ifndef NANO_SMALL
+    print1opt("-L", "--nonewlines",
+	N_("Don't add newlines to the ends of files"));
+#ifndef NANO_TINY
     print1opt("-N", "--noconvert",
 	N_("Don't convert files from DOS/Mac format"));
 #endif
@@ -1078,17 +765,17 @@ void usage(void)
 	N_("Quoting string"));
 #endif
     print1opt("-R", "--restricted", N_("Restricted mode"));
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     print1opt("-S", "--smooth", N_("Smooth scrolling"));
 #endif
     print1opt(_("-T [#cols]"), _("--tabsize=[#cols]"),
 	N_("Set width of a tab in cols to #cols"));
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     print1opt("-U", "--quickblank", N_("Do quick statusbar blanking"));
 #endif
     print1opt("-V", "--version",
 	N_("Print version information and exit"));
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     print1opt("-W", "--wordbounds",
 	N_("Detect word boundaries more accurately"));
 #endif
@@ -1099,7 +786,7 @@ void usage(void)
     print1opt("-c", "--const", N_("Constantly show cursor position"));
     print1opt("-d", "--rebinddelete",
 	N_("Fix Backspace/Delete confusion problem"));
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     print1opt("-i", "--autoindent",
 	N_("Automatically indent new lines"));
     print1opt("-k", "--cut", N_("Cut from cursor to end of line"));
@@ -1139,6 +826,9 @@ void usage(void)
     exit(0);
 }
 
+/* Display the current version of nano, the date and time it was
+ * compiled, contact information for it, and the configuration options
+ * it was compiled with. */
 void version(void)
 {
     printf(_(" GNU nano version %s (compiled %s, %s)\n"), VERSION,
@@ -1174,7 +864,7 @@ void version(void)
 #ifdef DISABLE_WRAPPING
     printf(" --disable-wrapping");
 #endif
-#ifdef DISABLE_ROOTWRAP
+#if defined(DISABLE_ROOTWRAP) && !defined(DISABLE_WRAPPING)
     printf(" --disable-wrapping-as-root");
 #endif
 #ifdef ENABLE_COLOR
@@ -1192,7 +882,7 @@ void version(void)
 #ifdef ENABLE_NANORC
     printf(" --enable-nanorc");
 #endif
-#ifdef NANO_SMALL
+#ifdef NANO_TINY
     printf(" --enable-tiny");
 #endif
 #ifdef ENABLE_UTF8
@@ -1204,78 +894,64 @@ void version(void)
     printf("\n");
 }
 
+/* Return 1 if the MORE_SPACE flag is set, and 0 otherwise.  This is
+ * used to calculate the relative screen position while taking this flag
+ * into account, since it adds one line to the edit window. */
 int no_more_space(void)
 {
     return ISSET(MORE_SPACE) ? 1 : 0;
 }
 
+/* Return 2 if the NO_HELP flag is set, and 0 otherwise.  This is used
+ * to calculate the relative screen position while taking this flag into
+ * account, since it removes two lines from the edit window. */
 int no_help(void)
 {
     return ISSET(NO_HELP) ? 2 : 0;
 }
 
+/* Indicate a disabled function on the statusbar. */
 void nano_disabled_msg(void)
 {
     statusbar(_("Sorry, support for this function has been disabled"));
 }
 
-void do_verbatim_input(void)
-{
-    int *kbinput;
-    size_t kbinput_len, i;
-    char *output;
-
-    statusbar(_("Verbatim Input"));
-
-    /* If constant cursor position display is on, make sure the current
-     * cursor position will be properly displayed on the statusbar. */
-    if (ISSET(CONST_UPDATE))
-	do_cursorpos(TRUE);
-
-    /* Read in all the verbatim characters. */
-    kbinput = get_verbatim_kbinput(edit, &kbinput_len);
-
-    /* Display all the verbatim characters at once, not filtering out
-     * control characters. */
-    output = charalloc(kbinput_len + 1);
-
-    for (i = 0; i < kbinput_len; i++)
-	output[i] = (char)kbinput[i];
-    output[i] = '\0';
-
-    do_output(output, kbinput_len, TRUE);
-
-    free(output);
-}
-
 void do_exit(void)
 {
     int i;
 
+    /* If the file hasn't been modified, pretend the user chose not to
+     * save. */
     if (!openfile->modified)
-	i = 0;		/* Pretend the user chose not to save. */
+	i = 0;
+    /* If the TEMP_FILE flag is set, pretend the user chose to save. */
     else if (ISSET(TEMP_FILE))
 	i = 1;
+    /* Otherwise, ask the user whether or not to save. */
     else
-	i = do_yesno(FALSE,
+	i = do_yesno_prompt(FALSE,
 		_("Save modified buffer (ANSWERING \"No\" WILL DESTROY CHANGES) ? "));
 
 #ifdef DEBUG
     dump_filestruct(openfile->fileage);
 #endif
 
+    /* If the user chose not to save, or if the user chose to save and
+     * the save succeeded, we're ready to exit. */
     if (i == 0 || (i == 1 && do_writeout(TRUE) == 0)) {
 #ifdef ENABLE_MULTIBUFFER
 	/* Exit only if there are no more open file buffers. */
 	if (!close_buffer())
 #endif
 	    finish();
+    /* If the user canceled, we go on. */
     } else if (i != 1)
 	statusbar(_("Cancelled"));
 
     display_main_list();
 }
 
+/* Initialize the signal handlers. */
 void signal_init(void)
 {
     /* Trap SIGINT and SIGQUIT because we want them to do useful
@@ -1290,7 +966,7 @@ void signal_init(void)
     sigaction(SIGHUP, &act, NULL);
     sigaction(SIGTERM, &act, NULL);
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     /* Trap SIGWINCH because we want to handle window resizes. */
     act.sa_handler = handle_sigwinch;
     sigaction(SIGWINCH, &act, NULL);
@@ -1309,19 +985,19 @@ void signal_init(void)
 	act.sa_handler = do_suspend;
 	sigaction(SIGTSTP, &act, NULL);
 
-	act.sa_handler = do_cont;
+	act.sa_handler = do_continue;
 	sigaction(SIGCONT, &act, NULL);
     }
 }
 
 /* Handler for SIGHUP (hangup) and SIGTERM (terminate). */
-void handle_hupterm(int signal)
+RETSIGTYPE handle_hupterm(int signal)
 {
     die(_("Received SIGHUP or SIGTERM\n"));
 }
 
 /* Handler for SIGTSTP (suspend). */
-void do_suspend(int signal)
+RETSIGTYPE do_suspend(int signal)
 {
     endwin();
     printf("\n\n\n\n\n%s\n", _("Use \"fg\" to return to nano"));
@@ -1341,9 +1017,9 @@ void do_suspend(int signal)
 }
 
 /* Handler for SIGCONT (continue after suspend). */
-void do_cont(int signal)
+RETSIGTYPE do_continue(int signal)
 {
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     /* Perhaps the user resized the window while we slept.  Handle it,
      * and update the screen in the process. */
     handle_sigwinch(0);
@@ -1353,8 +1029,9 @@ void do_cont(int signal)
 #endif
 }
 
-#ifndef NANO_SMALL
-void handle_sigwinch(int s)
+#ifndef NANO_TINY
+/* Handler for SIGWINCH (window size change). */
+RETSIGTYPE handle_sigwinch(int signal)
 {
     const char *tty = ttyname(0);
     int fd, result = 0;
@@ -1428,6 +1105,9 @@ void handle_sigwinch(int s)
     siglongjmp(jmpbuf, 1);
 }
 
+/* If allow is TRUE, block any SIGWINCH signals that we get, so that we
+ * can deal with them later.  If allow is FALSE, unblock any SIGWINCH
+ * signals that we have, so that we can deal with them now. */
 void allow_pending_sigwinch(bool allow)
 {
     sigset_t winch;
@@ -1435,9 +1115,10 @@ void allow_pending_sigwinch(bool allow)
     sigaddset(&winch, SIGWINCH);
     sigprocmask(allow ? SIG_UNBLOCK : SIG_BLOCK, &winch, NULL);
 }
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
+/* Handle the global toggle specified in which. */
 void do_toggle(const toggle *which)
 {
     bool enabled;
@@ -1486,8 +1167,10 @@ void do_toggle(const toggle *which)
     statusbar("%s %s", which->desc, enabled ? _("enabled") :
 	_("disabled"));
 }
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
 
+/* Disable extended input and output processing in our terminal
+ * settings. */
 void disable_extended_io(void)
 {
     struct termios term;
@@ -1498,6 +1181,8 @@ void disable_extended_io(void)
     tcsetattr(0, TCSANOW, &term);
 }
 
+/* Disable interpretation of the special control keys in our terminal
+ * settings. */
 void disable_signals(void)
 {
     struct termios term;
@@ -1507,7 +1192,9 @@ void disable_signals(void)
     tcsetattr(0, TCSANOW, &term);
 }
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
+/* Enable interpretation of the special control keys in our terminal
+ * settings. */
 void enable_signals(void)
 {
     struct termios term;
@@ -1518,6 +1205,8 @@ void enable_signals(void)
 }
 #endif
 
+/* Disable interpretation of the flow control characters in our terminal
+ * settings. */
 void disable_flow_control(void)
 {
     struct termios term;
@@ -1527,6 +1216,8 @@ void disable_flow_control(void)
     tcsetattr(0, TCSANOW, &term);
 }
 
+/* Enable interpretation of the flow control characters in our terminal
+ * settings. */
 void enable_flow_control(void)
 {
     struct termios term;
@@ -1555,6 +1246,15 @@ void terminal_init(void)
 	disable_flow_control();
 }
 
+/* Read in a character, interpret it as a shortcut or toggle if
+ * necessary, and return it.  Set meta_key to TRUE if the character is a
+ * meta sequence, set func_key to TRUE if the character is a function
+ * key, set s_or_t to TRUE if the character is a shortcut or toggle
+ * key, set ran_func to TRUE if we ran a function associated with a
+ * shortcut key, and set finished to TRUE if we're done after running
+ * or trying to run a function associated with a shortcut key.  If
+ * allow_funcs is FALSE, don't actually run any functions associated
+ * with shortcut keys. */
 int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 	*ran_func, bool *finished, bool allow_funcs)
 {
@@ -1566,7 +1266,7 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 	/* The length of the input buffer. */
     const shortcut *s;
     bool have_shortcut;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     const toggle *t;
     bool have_toggle;
 #endif
@@ -1597,7 +1297,7 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
     have_shortcut = (s != NULL || input == NANO_XON_KEY ||
 	input == NANO_XOFF_KEY || input == NANO_SUSPEND_KEY);
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     /* Check for a toggle in the main list. */
     t = get_toggle(input, *meta_key);
 
@@ -1608,7 +1308,7 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 
     /* Set s_or_t to TRUE if we got a shortcut or toggle. */
     *s_or_t = (have_shortcut
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	|| have_toggle
 #endif
 	);
@@ -1665,7 +1365,7 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 		case NANO_XOFF_KEY:
 		    statusbar(_("XOFF ignored, mumble mumble."));
 		    break;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 		case NANO_SUSPEND_KEY:
 		    if (ISSET(SUSPEND))
 			do_suspend(0);
@@ -1674,8 +1374,8 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 		/* Handle the normal edit window shortcuts, setting
 		 * ran_func to TRUE if we try to run their associated
 		 * functions and setting finished to TRUE to indicate
-		 * that we're done after trying to run their associated
-		 * functions. */
+		 * that we're done after running or trying to run their
+		 * associated functions. */
 		default:
 		    /* Blow away the text in the cutbuffer if we aren't
 		     * cutting text. */
@@ -1693,7 +1393,7 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 		    break;
 	    }
 	}
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	else if (have_toggle) {
 	    /* Blow away the text in the cutbuffer, since we aren't
 	     * cutting text. */
@@ -1713,6 +1413,7 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 }
 
 #ifndef DISABLE_MOUSE
+/* Handle a mouse click on the edit window or the shortcut list. */
 bool do_mouse(void)
 {
     int mouse_x, mouse_y;
@@ -1735,17 +1436,19 @@ bool do_mouse(void)
 
 	    /* Move to where the click occurred. */
 	    for (; openfile->current_y < mouse_y &&
-		openfile->current->next != NULL; openfile->current_y++)
+		openfile->current != openfile->filebot;
+		openfile->current_y++)
 		openfile->current = openfile->current->next;
 	    for (; openfile->current_y > mouse_y &&
-		openfile->current->prev != NULL; openfile->current_y--)
+		openfile->current != openfile->fileage;
+		openfile->current_y--)
 		openfile->current = openfile->current->prev;
 
 	    openfile->current_x = actual_x(openfile->current->data,
 		get_page_start(xplustabs()) + mouse_x);
 	    openfile->placewewant = xplustabs();
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    /* Clicking where the cursor is toggles the mark, as does
 	     * clicking beyond the line length with the cursor at the
 	     * end of the line. */
@@ -1802,9 +1505,10 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 	if (!allow_cntrls && is_cntrl_mbchar(output + i - char_buf_len))
 	    continue;
 
-	/* When a character is inserted on the current magicline, it
-	 * means we need a new one! */
-	if (openfile->filebot == openfile->current)
+	/* If the NO_NEWLINES flag isn't set, when a character is
+	 * added to the magicline, it means we need a new magicline! */
+	if (!ISSET(NO_NEWLINES) && openfile->filebot ==
+		openfile->current)
 	    new_magicline();
 
 	/* More dangerousness fun =) */
@@ -1813,17 +1517,17 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 
 	assert(openfile->current_x <= current_len);
 
-	charmove(&openfile->current->data[openfile->current_x +
-		char_buf_len],
-		&openfile->current->data[openfile->current_x],
-		current_len - openfile->current_x + char_buf_len);
-	strncpy(&openfile->current->data[openfile->current_x], char_buf,
+	charmove(openfile->current->data + openfile->current_x +
+		char_buf_len, openfile->current->data +
+		openfile->current_x, current_len - openfile->current_x +
+		char_buf_len);
+	strncpy(openfile->current->data + openfile->current_x, char_buf,
 		char_buf_len);
 	current_len += char_buf_len;
 	openfile->totsize++;
 	set_modified();
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	/* Note that current_x has not yet been incremented. */
 	if (openfile->mark_set && openfile->current ==
 		openfile->mark_begin && openfile->current_x <
@@ -1891,6 +1595,7 @@ int main(int argc, char **argv)
 	{"ignorercfiles", 0, NULL, 'I'},
 #endif
 	{"rebindkeypad", 0, NULL, 'K'},
+	{"nonewlines", 0, NULL, 'L'},
 	{"morespace", 0, NULL, 'O'},
 #ifndef DISABLE_JUSTIFY
 	{"quotestr", 1, NULL, 'Q'},
@@ -1924,7 +1629,7 @@ int main(int argc, char **argv)
 #endif
 	{"nohelp", 0, NULL, 'x'},
 	{"suspend", 0, NULL, 'z'},
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	{"smarthome", 0, NULL, 'A'},
 	{"backup", 0, NULL, 'B'},
 	{"backupdir", 1, NULL, 'C'},
@@ -1974,11 +1679,11 @@ int main(int argc, char **argv)
     while ((optchr =
 #ifdef HAVE_GETOPT_LONG
 	getopt_long(argc, argv,
-		"h?ABC:EFHIKNOQ:RST:UVWY:abcdefgijklmo:pr:s:tvwxz",
+		"h?ABC:EFHIKLNOQ:RST:UVWY:abcdefgijklmo:pr:s:tvwxz",
 		long_options, NULL)
 #else
 	getopt(argc, argv,
-		"h?ABC:EFHIKNOQ:RST:UVWY:abcdefgijklmo:pr:s:tvwxz")
+		"h?ABC:EFHIKLNOQ:RST:UVWY:abcdefgijklmo:pr:s:tvwxz")
 #endif
 		) != -1) {
 	switch (optchr) {
@@ -1990,7 +1695,7 @@ int main(int argc, char **argv)
 	    case 'j':
 		/* Pico compatibility flags. */
 		break;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    case 'A':
 		SET(SMART_HOME);
 		break;
@@ -2010,7 +1715,7 @@ int main(int argc, char **argv)
 		break;
 #endif
 #ifdef ENABLE_NANORC
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    case 'H':
 		SET(HISTORYLOG);
 		break;
@@ -2022,7 +1727,10 @@ int main(int argc, char **argv)
 	    case 'K':
 		SET(REBIND_KEYPAD);
 		break;
-#ifndef NANO_SMALL
+	    case 'L':
+		SET(NO_NEWLINES);
+		break;
+#ifndef NANO_TINY
 	    case 'N':
 		SET(NO_CONVERT);
 		break;
@@ -2038,7 +1746,7 @@ int main(int argc, char **argv)
 	    case 'R':
 		SET(RESTRICTED);
 		break;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    case 'S':
 		SET(SMOOTH_SCROLL);
 		break;
@@ -2050,7 +1758,7 @@ int main(int argc, char **argv)
 		    exit(1);
 		}
 		break;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    case 'U':
 		SET(QUICK_BLANK);
 		break;
@@ -2058,7 +1766,7 @@ int main(int argc, char **argv)
 	    case 'V':
 		version();
 		exit(0);
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    case 'W':
 		SET(WORD_BOUNDS);
 		break;
@@ -2074,7 +1782,7 @@ int main(int argc, char **argv)
 	    case 'd':
 		SET(REBIND_DELETE);
 		break;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    case 'i':
 		SET(AUTOINDENT);
 		break;
@@ -2160,7 +1868,7 @@ int main(int argc, char **argv)
 #ifndef DISABLE_WRAPJUSTIFY
 	ssize_t wrap_at_cpy = wrap_at;
 #endif
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	char *backup_dir_cpy = backup_dir;
 #endif
 #ifndef DISABLE_JUSTIFY
@@ -2175,7 +1883,7 @@ int main(int argc, char **argv)
 #ifndef DISABLE_OPERATINGDIR
 	operating_dir = NULL;
 #endif
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	backup_dir = NULL;
 #endif
 #ifndef DISABLE_JUSTIFY
@@ -2197,7 +1905,7 @@ int main(int argc, char **argv)
 	if (fill_used)
 	    wrap_at = wrap_at_cpy;
 #endif
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	if (backup_dir_cpy != NULL) {
 	    free(backup_dir);
 	    backup_dir = backup_dir_cpy;
@@ -2225,7 +1933,7 @@ int main(int argc, char **argv)
 #endif
 #endif /* ENABLE_NANORC */
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     history_init();
 #ifdef ENABLE_NANORC
     if (!ISSET(NO_RCFILE) && ISSET(HISTORYLOG))
@@ -2233,7 +1941,7 @@ int main(int argc, char **argv)
 #endif
 #endif
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     /* Set up the backup directory (unless we're using restricted mode,
      * in which case backups are disabled, since they would allow
      * reading from or writing to files not specified on the command
@@ -2295,7 +2003,7 @@ int main(int argc, char **argv)
     }
 #endif
 
-#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+#if !defined(NANO_TINY) && defined(ENABLE_NANORC)
     /* If whitespace wasn't specified, set its default value. */
     if (whitespace == NULL) {
 	whitespace = mallocstrcpy(NULL, "  ");
@@ -2411,7 +2119,7 @@ int main(int argc, char **argv)
 
     display_main_list();
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     /* Return here after a SIGWINCH. */
     sigsetjmp(jmpbuf, 1);
 #endif
@@ -2437,5 +2145,6 @@ int main(int argc, char **argv)
 		TRUE);
     }
 
+    /* We should never get here. */
     assert(FALSE);
 }
