@@ -1,8 +1,9 @@
-/* $Id: proto.h,v 1.280 2005/09/13 04:53:44 dolorous Exp $ */
+/* $Id: proto.h,v 1.316 2005/12/08 07:09:08 dolorous Exp $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
- *   Copyright (C) 1999-2005 Chris Allegretta                             *
+ *   Copyright (C) 1999-2004 Chris Allegretta                             *
+ *   Copyright (C) 2005 David Lawrence Ramsey                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -23,19 +24,31 @@
 #ifndef PROTO_H
 #define PROTO_H 1
 
-/* Externs. */
 #include "nano.h"
 
+/* Public externs.  See global.c for descriptions of them. */
 #ifndef DISABLE_WRAPJUSTIFY
 extern ssize_t fill;
 extern ssize_t wrap_at;
 #endif
-extern int editwinrows;
-extern long flags;
-extern ssize_t tabsize;
-extern int currslen;
 
-#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+extern char *last_search;
+extern char *last_replace;
+
+extern long flags;
+extern WINDOW *topwin;
+extern WINDOW *edit;
+extern WINDOW *bottomwin;
+extern int editwinrows;
+
+extern filestruct *cutbuffer;
+#ifndef DISABLE_JUSTIFY
+extern filestruct *jusbuffer;
+#endif
+extern partition *filepart;
+extern openfilestruct *openfile;
+
+#if !defined(NANO_TINY) && defined(ENABLE_NANORC)
 extern char *whitespace;
 extern int whitespace_len[2];
 #endif
@@ -53,45 +66,30 @@ extern size_t quotelen;
 #endif
 #endif
 
-#ifndef NANO_SMALL
+extern char *answer;
+
+extern ssize_t tabsize;
+
+#ifndef NANO_TINY
 extern char *backup_dir;
 #endif
-
-extern WINDOW *topwin, *edit, *bottomwin;
-extern char *answer;
-#ifndef DISABLE_HELP
-extern char *help_text;
-#endif
-extern char *last_search;
-extern char *last_replace;
 #ifndef DISABLE_OPERATINGDIR
 extern char *operating_dir;
 extern char *full_operating_dir;
 #endif
+
 #ifndef DISABLE_SPELLER
 extern char *alt_speller;
 #endif
 
-extern struct stat fileinfo;
-extern filestruct *cutbuffer;
-#ifndef DISABLE_JUSTIFY
-extern filestruct *jusbuffer;
-#endif
-extern partition *filepart;
-
-extern openfilestruct *openfile;
-
-#ifdef ENABLE_COLOR
-extern syntaxtype *syntaxes;
-extern char *syntaxstr;
-#endif
-
-extern shortcut *shortcut_list;
-extern shortcut *main_list, *whereis_list;
-extern shortcut *replace_list, *gotoline_list;
-extern shortcut *writefile_list, *insertfile_list;
+extern shortcut *main_list;
+extern shortcut *whereis_list;
+extern shortcut *replace_list;
 extern shortcut *replace_list_2;
-#ifndef NANO_SMALL
+extern shortcut *gotoline_list;
+extern shortcut *writefile_list;
+extern shortcut *insertfile_list;
+#ifndef NANO_TINY
 extern shortcut *extcmd_list;
 #endif
 #ifndef DISABLE_HELP
@@ -101,25 +99,21 @@ extern shortcut *help_list;
 extern shortcut *spell_list;
 #endif
 #ifndef DISABLE_BROWSER
-extern shortcut *browser_list, *gotodir_list;
+extern shortcut *browser_list;
+extern shortcut *gotodir_list;
+#endif
+
+#ifdef ENABLE_COLOR
+extern syntaxtype *syntaxes;
+extern char *syntaxstr;
 #endif
 
 extern const shortcut *currshortcut;
-
-#ifdef HAVE_REGEX_H
-extern regex_t search_regexp;
-extern regmatch_t regmatches[10];
-#ifdef ENABLE_COLOR
-extern regex_t syntaxfile_regexp;
-extern regmatch_t synfilematches[1];
-#endif /* ENABLE_COLOR */
-#endif /* HAVE_REGEX_H */
-
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 extern toggle *toggles;
 #endif
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 extern filestruct *search_history;
 extern filestruct *searchage;
 extern filestruct *searchbot;
@@ -128,11 +122,23 @@ extern filestruct *replaceage;
 extern filestruct *replacebot;
 #endif
 
+#ifdef HAVE_REGEX_H
+extern regex_t search_regexp;
+extern regmatch_t regmatches[10];
+#endif
+
 extern bool curses_ended;
 
 extern char *homedir;
 
-/* The functions we want available. */
+/* Public functions in browser.c. */
+#ifndef DISABLE_BROWSER
+char *do_browser(char *path, DIR *dir);
+char **browser_init(const char *path, int *longest, size_t *numents, DIR
+	*dir);
+char *do_browse_from(const char *inpath);
+void striponedir(char *path);
+#endif
 
 /* Public functions in chars.c. */
 #ifndef HAVE_ISBLANK
@@ -175,11 +181,11 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n);
 const char *nstrcasestr(const char *haystack, const char *needle);
 #endif
 const char *mbstrcasestr(const char *haystack, const char *needle);
-#if !defined(NANO_SMALL) || !defined(DISABLE_TABCOMP)
+#if !defined(NANO_TINY) || !defined(DISABLE_TABCOMP)
 const char *revstrstr(const char *haystack, const char *needle, const
 	char *rev_start);
 #endif
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 const char *revstrcasestr(const char *haystack, const char *needle,
 	const char *rev_start);
 const char *mbrevstrcasestr(const char *haystack, const char *needle,
@@ -190,6 +196,10 @@ size_t mbstrlen(const char *s);
 size_t nstrnlen(const char *s, size_t maxlen);
 #endif
 size_t mbstrnlen(const char *s, size_t maxlen);
+#ifndef NANO_TINY
+char *revstrpbrk(const char *s, const char *accept, const char
+	*rev_start);
+#endif
 #ifndef DISABLE_JUSTIFY
 char *mbstrchr(const char *s, char *c);
 #ifdef ENABLE_NANORC
@@ -209,17 +219,17 @@ bool is_valid_mbstring(const char *s);
 void set_colorpairs(void);
 void color_init(void);
 void color_update(void);
-#endif /* ENABLE_COLOR */
+#endif
 
 /* Public functions in cut.c. */
 void cutbuffer_reset(void);
 void cut_line(void);
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 void cut_marked(void);
 void cut_to_eol(void);
 #endif
 void do_cut_text(void);
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 void do_cut_till_end(void);
 #endif
 void do_uncut_text(void);
@@ -229,6 +239,9 @@ void make_new_buffer(void);
 void initialize_buffer(void);
 void initialize_buffer_text(void);
 void open_buffer(const char *filename);
+#ifndef DISABLE_SPELLER
+void replace_buffer(const char *filename);
+#endif
 void display_buffer(void);
 #ifdef ENABLE_MULTIBUFFER
 void switch_to_prevnext_buffer(bool next);
@@ -242,7 +255,7 @@ void read_file(FILE *f, const char *filename);
 int open_file(const char *filename, bool newfie, FILE **f);
 char *get_next_filename(const char *name, const char *suffix);
 void do_insertfile(
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	bool execute
 #else
 	void
@@ -256,13 +269,13 @@ char *safe_tempfile(FILE **f);
 void init_operating_dir(void);
 bool check_operating_dir(const char *currpath, bool allow_tabcomp);
 #endif
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 void init_backup_dir(void);
 #endif
 int copy_file(FILE *inn, FILE *out);
 int write_file(const char *name, FILE *f_open, bool tmp, append_type
 	append, bool nonamechange);
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 int write_marked_file(const char *name, FILE *f_open, bool tmp,
 	append_type append);
 #endif
@@ -281,14 +294,7 @@ char **cwd_tab_completion(const char *buf, size_t *num_matches, size_t
 char *input_tab(char *buf, size_t *place, bool *lastwastab, bool *list);
 #endif
 const char *tail(const char *foo);
-#ifndef DISABLE_BROWSER
-void striponedir(char *path);
-char **browser_init(const char *path, int *longest, size_t *numents, DIR
-	*dir);
-char *do_browser(char *path, DIR *dir);
-char *do_browse_from(const char *inpath);
-#endif
-#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+#if !defined(NANO_TINY) && defined(ENABLE_NANORC)
 char *histfilename(void);
 void load_history(void);
 bool writehist(FILE *hist, filestruct *histhead);
@@ -297,7 +303,7 @@ void save_history(void);
 
 /* Public functions in global.c. */
 size_t length_of_list(const shortcut *s);
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 void toggle_init_one(int val, const char *desc, long flag);
 void toggle_init(void);
 #endif
@@ -313,6 +319,13 @@ void free_shortcutage(shortcut **shortcutage);
 void thanks_for_all_the_fish(void);
 #endif
 
+/* Public functions in help.c. */
+#ifndef DISABLE_HELP
+void do_help(void);
+void help_init(void);
+size_t help_line_len(const char *ptr);
+#endif
+
 /* Public functions in move.c. */
 void do_first_line(void);
 void do_last_line(void);
@@ -324,7 +337,7 @@ void do_para_begin_void(void);
 void do_para_end(bool allow_update);
 void do_para_end_void(void);
 #endif
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 bool do_next_word(bool allow_punct, bool allow_update);
 void do_next_word_void(void);
 bool do_prev_word(bool allow_punct, bool allow_update);
@@ -333,7 +346,13 @@ void do_prev_word_void(void);
 void do_home(void);
 void do_end(void);
 void do_up(void);
+#ifndef NANO_TINY
+void do_scroll_up(void);
+#endif
 void do_down(void);
+#ifndef NANO_TINY
+void do_scroll_down(void);
+#endif
 void do_left(void);
 void do_right(void);
 
@@ -369,9 +388,6 @@ void window_init(void);
 #ifndef DISABLE_MOUSE
 void mouse_init(void);
 #endif
-#ifndef DISABLE_HELP
-void help_init(void);
-#endif
 void print1opt_full(const char *shortflag
 #ifdef HAVE_GETOPT_LONG
 	, const char *longflag
@@ -382,22 +398,21 @@ void version(void);
 int no_more_space(void);
 int no_help(void);
 void nano_disabled_msg(void);
-void do_verbatim_input(void);
 void do_exit(void);
 void signal_init(void);
-void handle_hupterm(int signal);
-void do_suspend(int signal);
-void do_cont(int signal);
-#ifndef NANO_SMALL
-void handle_sigwinch(int s);
+RETSIGTYPE handle_hupterm(int signal);
+RETSIGTYPE do_suspend(int signal);
+RETSIGTYPE do_continue(int signal);
+#ifndef NANO_TINY
+RETSIGTYPE handle_sigwinch(int signal);
 void allow_pending_sigwinch(bool allow);
 #endif
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 void do_toggle(const toggle *which);
 #endif
 void disable_extended_io(void);
 void disable_signals(void);
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 void enable_signals(void);
 #endif
 void disable_flow_control(void);
@@ -410,6 +425,54 @@ bool do_mouse(void);
 #endif
 void do_output(char *output, size_t output_len, bool allow_cntrls);
 
+/* Public functions in prompt.c. */
+int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
+	bool *ran_func, bool *finished, bool allow_funcs);
+#ifndef DISABLE_MOUSE
+bool do_statusbar_mouse(void);
+#endif
+void do_statusbar_output(char *output, size_t output_len, bool
+	*got_enter, bool allow_cntrls);
+void do_statusbar_home(void);
+void do_statusbar_end(void);
+void do_statusbar_left(void);
+void do_statusbar_right(void);
+void do_statusbar_backspace(void);
+void do_statusbar_delete(void);
+void do_statusbar_cut_text(void);
+#ifndef NANO_TINY
+bool do_statusbar_next_word(bool allow_punct);
+bool do_statusbar_prev_word(bool allow_punct);
+#endif
+void do_statusbar_verbatim_input(bool *got_enter);
+#ifndef NANO_TINY
+bool find_statusbar_bracket_match(bool reverse, const char
+	*bracket_set);
+void do_statusbar_find_bracket(void);
+#endif
+size_t statusbar_xplustabs(void);
+size_t get_statusbar_page_start(size_t start_col, size_t column);
+void update_statusbar_line(const char *curranswer, size_t index);
+void reset_statusbar_cursor(void);
+bool need_statusbar_horizontal_update(size_t old_pww);
+int get_prompt_string(bool allow_tabs, const char *curranswer,
+#ifndef NANO_TINY
+	filestruct **history_list,
+#endif
+	const shortcut *s
+#ifndef DISABLE_TABCOMP
+	, bool *list
+#endif
+	);
+int do_prompt(bool allow_tabs, const shortcut *s, const char
+	*curranswer,
+#ifndef NANO_TINY
+	filestruct **history_list,
+#endif
+	const char *msg, ...);
+void do_prompt_abort(void);
+int do_yesno_prompt(bool all, const char *msg);
+
 /* Public functions in rcfile.c. */
 #ifdef ENABLE_NANORC
 void rcfile_error(const char *msg, ...);
@@ -421,10 +484,10 @@ char *parse_next_regex(char *ptr);
 bool nregcomp(const char *regex, int eflags);
 void parse_syntax(char *ptr);
 void parse_colors(char *ptr, bool icase);
-#endif /* ENABLE_COLOR */
+#endif
 void parse_rcfile(FILE *rcstream);
 void do_rcfile(void);
-#endif /* ENABLE_NANORC */
+#endif
 
 /* Public functions in search.c. */
 #ifdef HAVE_REGEX_H
@@ -432,37 +495,41 @@ int regexp_init(const char *regexp);
 void regexp_cleanup(void);
 #endif
 void not_found_msg(const char *str);
-void search_abort(void);
+void search_replace_abort(void);
 void search_init_globals(void);
 int search_init(bool replacing, bool use_answer);
-bool findnextstr(bool can_display_wrap, bool wholeword, bool
-	no_sameline, const filestruct *begin, size_t begin_x, const char
-	*needle, size_t *needle_len);
+bool findnextstr(
+#ifndef DISABLE_SPELLER
+	bool whole_word,
+#endif
+	bool no_sameline, const filestruct *begin, size_t begin_x, const
+	char *needle, size_t *needle_len);
 void findnextstr_wrap_reset(void);
 void do_search(void);
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 void do_research(void);
 #endif
-void replace_abort(void);
 #ifdef HAVE_REGEX_H
 int replace_regexp(char *string, bool create);
 #endif
 char *replace_line(const char *needle);
-ssize_t do_replace_loop(const char *needle, const filestruct
-	*real_current, size_t *real_current_x, bool wholewords, bool
-	*canceled);
+ssize_t do_replace_loop(
+#ifndef DISABLE_SPELLER
+	bool whole_word,
+#endif
+	bool *canceled, const filestruct *real_current, size_t
+	*real_current_x, const char *needle);
 void do_replace(void);
 void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 	bool interactive, bool save_pos, bool allow_update);
 void do_gotolinecolumn_void(void);
 #ifndef DISABLE_SPELLER
-void do_gotopos(ssize_t line, size_t pos_x, ssize_t pos_y, size_t
+void do_gotopos(ssize_t pos_line, size_t pos_x, ssize_t pos_y, size_t
 	pos_pww);
 #endif
-#ifndef NANO_SMALL
-#ifdef HAVE_REGEX_H
+#ifndef NANO_TINY
+bool find_bracket_match(bool reverse, const char *bracket_set);
 void do_find_bracket(void);
-#endif
 #ifdef ENABLE_NANORC
 bool history_has_changed(void);
 #endif
@@ -476,28 +543,32 @@ char *get_history_newer(filestruct **h);
 #ifndef DISABLE_TABCOMP
 char *get_history_completion(filestruct **h, const char *s, size_t len);
 #endif
-#endif /* !NANO_SMALL */
+#endif
 
 /* Public functions in text.c. */
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 void do_mark(void);
 #endif
 void do_delete(void);
 void do_backspace(void);
 void do_tab(void);
 void do_enter(void);
-#ifndef NANO_SMALL
-void cancel_command(int signal);
+#ifndef NANO_TINY
+RETSIGTYPE cancel_command(int signal);
 bool execute_command(const char *command);
 #endif
 #ifndef DISABLE_WRAPPING
 void wrap_reset(void);
 bool do_wrap(filestruct *line);
 #endif
-#if !defined(DISABLE_HELP) || !defined(DISABLE_JUSTIFY) || !defined(DISABLE_WRAPPING)
-ssize_t break_line(const char *line, ssize_t goal, bool newline);
+#if !defined(DISABLE_HELP) || !defined(DISABLE_WRAPJUSTIFY)
+ssize_t break_line(const char *line, ssize_t goal
+#ifndef DISABLE_HELP
+	, bool newline
 #endif
-#if !defined(NANO_SMALL) || !defined(DISABLE_JUSTIFY)
+	);
+#endif
+#if !defined(NANO_TINY) || !defined(DISABLE_JUSTIFY)
 size_t indent_length(const char *line);
 #endif
 #ifndef DISABLE_JUSTIFY
@@ -509,22 +580,22 @@ bool indents_match(const char *a_line, size_t a_indent, const char
 	*b_line, size_t b_indent);
 bool begpar(const filestruct *const foo);
 bool inpar(const filestruct *const foo);
-filestruct *backup_lines(filestruct *first_line, size_t par_len, size_t
-	quote_len);
+void backup_lines(filestruct *first_line, size_t par_len);
 bool find_paragraph(size_t *const quote, size_t *const par);
 void do_justify(bool full_justify);
 void do_justify_void(void);
 void do_full_justify(void);
-#endif /* !DISABLE_JUSTIFY */
+#endif
 #ifndef DISABLE_SPELLER
 bool do_int_spell_fix(const char *word);
 const char *do_int_speller(const char *tempfile_name);
 const char *do_alt_speller(char *tempfile_name);
 void do_spell(void);
 #endif
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 void do_wordlinechar_count(void);
 #endif
+void do_verbatim_input(void);
 
 /* Public functions in utils.c. */
 int digits(size_t n);
@@ -535,22 +606,24 @@ void align(char **str);
 void null_at(char **data, size_t index);
 void unsunder(char *str, size_t true_len);
 void sunder(char *str);
-#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+#if !defined(NANO_TINY) && defined(ENABLE_NANORC)
 #ifndef HAVE_GETLINE
 ssize_t ngetline(char **lineptr, size_t *n, FILE *stream);
 #endif
 #ifndef HAVE_GETDELIM
 ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream);
 #endif
-#endif /* !NANO_SMALL && ENABLE_NANORC */
+#endif
 #ifdef HAVE_REGEX_H
 #ifdef BROKEN_REGEXEC
 int safe_regexec(const regex_t *preg, const char *string, size_t nmatch,
 	regmatch_t pmatch[], int eflags);
 #endif
-int regexp_bol_or_eol(const regex_t *preg, const char *string);
+bool regexp_bol_or_eol(const regex_t *preg, const char *string);
 #endif
+#ifndef DISABLE_SPELLER
 bool is_whole_word(size_t pos, const char *buf, const char *word);
+#endif
 const char *strstrwrapper(const char *haystack, const char *needle,
 	const char *start);
 void nperror(const char *s);
@@ -559,16 +632,28 @@ void *nrealloc(void *ptr, size_t howmuch);
 char *mallocstrncpy(char *dest, const char *src, size_t n);
 char *mallocstrcpy(char *dest, const char *src);
 char *mallocstrassn(char *dest, char *src);
+size_t get_page_start(size_t column);
+size_t xplustabs(void);
+size_t actual_x(const char *s, size_t column);
+size_t strnlenpt(const char *s, size_t maxlen);
+size_t strlenpt(const char *s);
 void new_magicline(void);
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 void remove_magicline(void);
 void mark_order(const filestruct **top, size_t *top_x, const filestruct
 	**bot, size_t *bot_x, bool *right_side_up);
 #endif
 size_t get_totsize(const filestruct *begin, const filestruct *end);
+#ifndef NDEBUG
+int check_linenumbers(const filestruct *fileptr);
+#endif
+#ifdef DEBUG
+void dump_filestruct(const filestruct *inptr);
+void dump_filestruct_reverse(void);
+#endif
 
 /* Public functions in winio.c. */
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 void reset_kbinput(void);
 #endif
 void get_key_buffer(WINDOW *win);
@@ -578,7 +663,7 @@ void unget_kbinput(int kbinput, bool meta_key, bool func_key);
 int *get_input(WINDOW *win, size_t input_len);
 int get_kbinput(WINDOW *win, bool *meta_key, bool *func_key);
 int parse_kbinput(WINDOW *win, bool *meta_key, bool *func_key
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	, bool reset
 #endif
 	);
@@ -586,12 +671,12 @@ int get_escape_seq_kbinput(const int *seq, size_t seq_len, bool
 	*ignore_seq);
 int get_escape_seq_abcd(int kbinput);
 int get_byte_kbinput(int kbinput
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	, bool reset
 #endif
 	);
 long get_unicode_kbinput(int kbinput
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	, bool reset
 #endif
 	);
@@ -604,32 +689,9 @@ bool get_mouseinput(int *mouse_x, int *mouse_y, bool allow_shortcuts);
 #endif
 const shortcut *get_shortcut(const shortcut *s_list, int *kbinput, bool
 	*meta_key, bool *func_key);
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 const toggle *get_toggle(int kbinput, bool meta_key);
 #endif
-int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
-	bool *ran_func, bool *finished, bool allow_funcs);
-#ifndef DISABLE_MOUSE
-bool do_statusbar_mouse(void);
-#endif
-void do_statusbar_output(char *output, size_t output_len, bool
-	*got_enter, bool allow_cntrls);
-void do_statusbar_home(void);
-void do_statusbar_end(void);
-void do_statusbar_right(void);
-void do_statusbar_left(void);
-void do_statusbar_backspace(void);
-void do_statusbar_delete(void);
-void do_statusbar_cut_text(void);
-#ifndef NANO_SMALL
-bool do_statusbar_next_word(bool allow_punct);
-bool do_statusbar_prev_word(bool allow_punct);
-#endif
-void do_statusbar_verbatim_input(bool *got_enter);
-size_t xplustabs(void);
-size_t actual_x(const char *str, size_t xplus);
-size_t strnlenpt(const char *buf, size_t size);
-size_t strlenpt(const char *buf);
 void blank_line(WINDOW *win, int y, int x, int n);
 void blank_titlebar(void);
 void blank_topbar(void);
@@ -639,31 +701,14 @@ void blank_bottombars(void);
 void check_statusblank(void);
 char *display_string(const char *buf, size_t start_col, size_t len, bool
 	dollars);
-void nanoget_repaint(const char *buf, const char *inputbuf, size_t x);
-int nanogetstr(bool allow_tabs, const char *buf, const char *curranswer,
-#ifndef NANO_SMALL
-	filestruct **history_list,
-#endif
-	const shortcut *s
-#ifndef DISABLE_TABCOMP
-	, bool *list
-#endif
-	);
-int statusq(bool allow_tabs, const shortcut *s, const char *curranswer,
-#ifndef NANO_SMALL
-	filestruct **history_list,
-#endif
-	const char *msg, ...);
-void statusq_abort(void);
 void titlebar(const char *path);
 void set_modified(void);
 void statusbar(const char *msg, ...);
 void bottombars(const shortcut *s);
 void onekey(const char *keystroke, const char *desc, size_t len);
-size_t get_page_start(size_t column);
 void reset_cursor(void);
-void edit_add(const filestruct *fileptr, const char *converted, int
-	yval, size_t start);
+void edit_draw(const filestruct *fileptr, const char *converted, int
+	line, size_t start);
 void update_line(const filestruct *fileptr, size_t index);
 bool need_horizontal_update(size_t old_pww);
 bool need_vertical_update(size_t old_pww);
@@ -671,24 +716,12 @@ void edit_scroll(scroll_dir direction, ssize_t nlines);
 void edit_redraw(const filestruct *old_current, size_t old_pww);
 void edit_refresh(void);
 void edit_update(update_type location);
-int do_yesno(bool all, const char *msg);
 void total_redraw(void);
 void total_refresh(void);
 void display_main_list(void);
 void do_cursorpos(bool constant);
 void do_cursorpos_void(void);
-#ifndef DISABLE_HELP
-size_t help_line_len(const char *ptr);
-void do_help(void);
-#endif
 void do_replace_highlight(bool highlight, const char *word);
-#ifndef NDEBUG
-int check_linenumbers(const filestruct *fileptr);
-#endif
-#ifdef DEBUG
-void dump_filestruct(const filestruct *inptr);
-void dump_filestruct_reverse(void);
-#endif
 #ifdef NANO_EXTRA
 void do_credits(void);
 #endif
