@@ -1,8 +1,9 @@
-/* $Id: rcfile.c,v 1.83 2005/08/10 22:12:28 dolorous Exp $ */
+/* $Id: rcfile.c,v 1.91 2005/12/08 07:24:54 dolorous Exp $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
- *   Copyright (C) 1999-2005 Chris Allegretta                             *
+ *   Copyright (C) 2001-2004 Chris Allegretta                             *
+ *   Copyright (C) 2005 David Lawrence Ramsey                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -20,9 +21,7 @@
  *                                                                        *
  **************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "proto.h"
 
 #include <stdarg.h>
 #include <string.h>
@@ -30,7 +29,6 @@
 #include <errno.h>
 #include <unistd.h>
 #include <ctype.h>
-#include "proto.h"
 
 #ifdef ENABLE_NANORC
 
@@ -51,6 +49,7 @@ const static rcoption rcopts[] = {
     {"morespace", MORE_SPACE},
     {"nofollow", NOFOLLOW_SYMLINKS},
     {"nohelp", NO_HELP},
+    {"nonewlines", NO_NEWLINES},
 #ifndef DISABLE_WRAPPING
     {"nowrap", NO_WRAP},
 #endif
@@ -74,7 +73,7 @@ const static rcoption rcopts[] = {
     {"tabsize", 0},
     {"tempfile", TEMP_FILE},
     {"view", VIEW_MODE},
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     {"autoindent", AUTOINDENT},
     {"backup", BACKUP_FILE},
     {"backupdir", 0},
@@ -94,8 +93,12 @@ const static rcoption rcopts[] = {
 };
 
 static bool errors = FALSE;
+	/* Whether we got any errors while parsing an rcfile. */
 static size_t lineno = 0;
+	/* If we did, the line number where the current error
+	 * occurred. */
 static char *nanorc = NULL;
+	/* The path to the rcfile we're parsing. */
 #ifdef ENABLE_COLOR
 static syntaxtype *endsyntax = NULL;
 	/* The end of the list of syntaxes. */
@@ -104,7 +107,7 @@ static colortype *endcolor = NULL;
 #endif
 
 /* We have an error in some part of the rcfile.  Put it on stderr and
- * make the user hit Return to continue starting up nano. */
+ * make the user hit Enter to continue starting up nano. */
 void rcfile_error(const char *msg, ...)
 {
     va_list ap;
@@ -181,6 +184,8 @@ char *parse_argument(char *ptr)
 }
 
 #ifdef ENABLE_COLOR
+/* Return the short value corresponding to the color named in colorname,
+ * and set bright to TRUE if that color is bright. */
 short color_to_short(const char *colorname, bool *bright)
 {
     short mcolor = -1;
@@ -218,6 +223,7 @@ short color_to_short(const char *colorname, bool *bright)
     return mcolor;
 }
 
+/* Parse the next regex string from the line at ptr, and return it. */
 char *parse_next_regex(char *ptr)
 {
     assert(ptr != NULL);
@@ -266,6 +272,8 @@ bool nregcomp(const char *regex, int eflags)
     return (rc == 0);
 }
 
+/* Parse the next syntax string from the line at ptr, and add it to the
+ * global list of color syntaxes. */
 void parse_syntax(char *ptr)
 {
     const char *fileregptr = NULL, *nameptr = NULL;
@@ -373,8 +381,9 @@ void parse_syntax(char *ptr)
     }
 }
 
-/* Parse the color stuff into the colorstrings array.  If icase is TRUE,
- * treat the color stuff as case insensitive. */
+/* Parse the color string in the line at ptr, and add it to the current
+ * file's associated colors.  If icase is TRUE, treat the color string
+ * as case insensitive. */
 void parse_colors(char *ptr, bool icase)
 {
     short fg, bg;
@@ -530,7 +539,8 @@ void parse_colors(char *ptr, bool icase)
 }
 #endif /* ENABLE_COLOR */
 
-/* Parse the rcfile, once it has been opened successfully. */
+/* Parse the rcfile, once it has been opened successfully at
+ * rcstream. */
 void parse_rcfile(FILE *rcstream)
 {
     char *buf = NULL;
@@ -638,7 +648,7 @@ void parse_rcfile(FILE *rcstream)
 				free(option);
 			} else
 #endif
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 			if (strcasecmp(rcopts[i].name, "whitespace") == 0) {
 			    whitespace = option;
 			    if (mbstrlen(whitespace) != 2 ||
@@ -680,7 +690,7 @@ void parse_rcfile(FILE *rcstream)
 			    quotestr = option;
 			else
 #endif
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 			if (strcasecmp(rcopts[i].name,
 				"backupdir") == 0)
 			    backup_dir = option;
@@ -725,7 +735,7 @@ void parse_rcfile(FILE *rcstream)
 
     if (errors) {
 	errors = FALSE;
-	fprintf(stderr, _("\nPress Return to continue starting nano\n"));
+	fprintf(stderr, _("\nPress Enter to continue starting nano\n"));
 	while (getchar() != '\n')
 	    ;
     }
@@ -733,14 +743,13 @@ void parse_rcfile(FILE *rcstream)
     return;
 }
 
-/* The main rc file function, tries to open the rc file */
+/* The main rcfile function.  It tries to open the system-wide rcfile,
+ * followed by the local rcfile. */
 void do_rcfile(void)
 {
     FILE *rcstream;
 
 #ifdef SYSCONFDIR
-    assert(sizeof(SYSCONFDIR) == strlen(SYSCONFDIR) + 1);
-
     nanorc = mallocstrcpy(nanorc, SYSCONFDIR "/nanorc");
     /* Try to open the system-wide nanorc. */
     rcstream = fopen(nanorc, "r");
