@@ -1,8 +1,9 @@
-/* $Id: search.c,v 1.161 2005/08/11 18:30:47 dolorous Exp $ */
+/* $Id: search.c,v 1.178 2005/12/08 07:09:08 dolorous Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
- *   Copyright (C) 2000-2005 Chris Allegretta                             *
+ *   Copyright (C) 1999-2004 Chris Allegretta                             *
+ *   Copyright (C) 2005 David Lawrence Ramsey                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -20,20 +21,17 @@
  *                                                                        *
  **************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "proto.h"
 
 #include <string.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <ctype.h>
 #include <errno.h>
-#include "proto.h"
 
 static bool search_last_line = FALSE;
 	/* Have we gone past the last line while searching? */
-#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+#if !defined(NANO_TINY) && defined(ENABLE_NANORC)
 static bool history_changed = FALSE;
 	/* Have any of the history lists changed? */
 #endif
@@ -49,7 +47,7 @@ static bool regexp_compiled = FALSE;
 int regexp_init(const char *regexp)
 {
     int rc = regcomp(&search_regexp, regexp, REG_EXTENDED
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	| (ISSET(CASE_SENSITIVE) ? 0 : REG_ICASE)
 #endif
 	);
@@ -70,6 +68,8 @@ int regexp_init(const char *regexp)
     return 1;
 }
 
+/* Decompile the compiled regular expression we used in the last
+ * search, if any. */
 void regexp_cleanup(void)
 {
     if (regexp_compiled) {
@@ -79,6 +79,8 @@ void regexp_cleanup(void)
 }
 #endif
 
+/* Indicate on the statusbar that the string at str was not found by the
+ * last search. */
 void not_found_msg(const char *str)
 {
     char *disp;
@@ -95,10 +97,14 @@ void not_found_msg(const char *str)
     free(disp);
 }
 
-void search_abort(void)
+/* Abort the current search or replace.  Clean up by displaying the main
+ * shortcut list, updating the screen if the mark was on before, and
+ * decompiling the compiled regular expression we used in the last
+ * search, if any. */
+void search_replace_abort(void)
 {
     display_main_list();
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     if (openfile->mark_set)
 	edit_refresh();
 #endif
@@ -107,6 +113,7 @@ void search_abort(void)
 #endif
 }
 
+/* Initialize the global search and replace strings. */
 void search_init_globals(void)
 {
     if (last_search == NULL)
@@ -155,20 +162,20 @@ int search_init(bool replacing, bool use_answer)
 	/* We use (COLS / 3) here because we need to see more on the
 	 * line. */
 	sprintf(buf, " [%s%s]", disp,
-		strlenpt(last_search) > COLS / 3 ? "..." : "");
+		(strlenpt(last_search) > COLS / 3) ? "..." : "");
 	free(disp);
     } else
 	buf = mallocstrcpy(NULL, "");
 
     /* This is now one simple call.  It just does a lot. */
-    i = statusq(FALSE, replacing ? replace_list : whereis_list,
+    i = do_prompt(FALSE, replacing ? replace_list : whereis_list,
 	backupstring,
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	&search_history,
 #endif
 	"%s%s%s%s%s%s", _("Search"),
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	/* This string is just a modifier for the search prompt; no
 	 * grammar is implied. */
 	ISSET(CASE_SENSITIVE) ? _(" [Case Sensitive]") :
@@ -182,7 +189,7 @@ int search_init(bool replacing, bool use_answer)
 #endif
 		"",
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	/* This string is just a modifier for the search prompt; no
 	 * grammar is implied. */
 	ISSET(BACKWARDS_SEARCH) ? _(" [Backwards]") :
@@ -190,7 +197,7 @@ int search_init(bool replacing, bool use_answer)
 		"",
 
 	replacing ?
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 		openfile->mark_set ? _(" (to replace) in selection") :
 #endif
 		_(" (to replace)") : "",
@@ -221,7 +228,7 @@ int search_init(bool replacing, bool use_answer)
 		    return -1;
 #endif
 		break;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    case TOGGLE_CASE_KEY:
 		TOGGLE(CASE_SENSITIVE);
 		backupstring = mallocstrcpy(backupstring, answer);
@@ -256,21 +263,22 @@ int search_init(bool replacing, bool use_answer)
 
 /* Look for needle, starting at (current, current_x).  If no_sameline is
  * TRUE, skip over begin when looking for needle.  begin is the line
- * where we first started searching, at column begin_x.  If
- * can_display_wrap is TRUE, we put messages on the statusbar, wrap
- * around the file boundaries.  The return value specifies whether we
- * found anything.  If we did, set needle_len to the length of the
- * string we found if it isn't NULL. */
-bool findnextstr(bool can_display_wrap, bool wholeword, bool
-	no_sameline, const filestruct *begin, size_t begin_x, const char
-	*needle, size_t *needle_len)
+ * where we first started searching, at column begin_x.  The return
+ * value specifies whether we found anything.  If we did, set needle_len
+ * to the length of the string we found if it isn't NULL. */
+bool findnextstr(
+#ifndef DISABLE_SPELLER
+	bool whole_word,
+#endif
+	bool no_sameline, const filestruct *begin, size_t begin_x, const
+	char *needle, size_t *needle_len)
 {
     filestruct *fileptr = openfile->current;
     const char *rev_start = NULL, *found = NULL;
     size_t found_len;
 	/* The length of the match we found. */
     size_t current_x_find = 0;
-	/* The location of the match we found. */
+	/* The location in the current line of the match we found. */
     ssize_t current_y_find = openfile->current_y;
 
     /* rev_start might end up 1 character before the start or after the
@@ -279,20 +287,22 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
      * rev_start will be properly set when the search continues on the
      * previous or next line. */
     rev_start =
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	ISSET(BACKWARDS_SEARCH) ?
 	fileptr->data + (openfile->current_x - 1) :
 #endif
 	fileptr->data + (openfile->current_x + 1);
 
-    /* Look for needle in searchstr. */
+    /* Look for needle in the current line we're searching. */
     while (TRUE) {
 	found = strstrwrapper(fileptr->data, needle, rev_start);
 
 	/* We've found a potential match. */
 	if (found != NULL) {
+#ifndef DISABLE_SPELLER
 	    bool found_whole = FALSE;
 		/* Is this potential match a whole word? */
+#endif
 
 	    /* Set found_len to the length of the potential match. */
 	    found_len =
@@ -302,9 +312,10 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
 #endif
 		strlen(needle);
 
+#ifndef DISABLE_SPELLER
 	    /* If we're searching for whole words, see if this potential
 	     * match is a whole word. */
-	    if (wholeword) {
+	    if (whole_word) {
 		char *word = mallocstrncpy(NULL, found, found_len + 1);
 		word[found_len] = '\0';
 
@@ -312,24 +323,27 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
 			fileptr->data, word);
 		free(word);
 	    }
+#endif
 
 	    /* If we're searching for whole words and this potential
 	     * match isn't a whole word, or if we're not allowed to find
 	     * a match on the same line we started on and this potential
 	     * match is on that line, continue searching. */
-	    if ((!wholeword || found_whole) && (!no_sameline ||
-		fileptr != openfile->current))
+	    if (
+#ifndef DISABLE_SPELLER
+		(!whole_word || found_whole) &&
+#endif
+		(!no_sameline || fileptr != openfile->current))
 		break;
 	}
 
 	/* We've finished processing the file, so get out. */
 	if (search_last_line) {
-	    if (can_display_wrap)
-		not_found_msg(needle);
+	    not_found_msg(needle);
 	    return FALSE;
 	}
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	if (ISSET(BACKWARDS_SEARCH)) {
 	    fileptr = fileptr->prev;
 	    current_y_find--;
@@ -337,16 +351,14 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
 #endif
 	    fileptr = fileptr->next;
 	    current_y_find++;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	}
 #endif
 
-	/* Start or end of buffer reached, so wrap around. */
+	/* We've reached the start or end of the buffer, so wrap
+	 * around. */
 	if (fileptr == NULL) {
-	    if (!can_display_wrap)
-		return FALSE;
-
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    if (ISSET(BACKWARDS_SEARCH)) {
 		fileptr = openfile->filebot;
 		current_y_find = editwinrows - 1;
@@ -354,20 +366,18 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
 #endif
 		fileptr = openfile->fileage;
 		current_y_find = 0;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    }
 #endif
-
-	    if (can_display_wrap)
-		statusbar(_("Search Wrapped"));
+	    statusbar(_("Search Wrapped"));
 	}
 
-	/* Original start line reached. */
+	/* We've reached the original starting line. */
 	if (fileptr == begin)
 	    search_last_line = TRUE;
 
 	rev_start = fileptr->data;
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	if (ISSET(BACKWARDS_SEARCH))
 	    rev_start += strlen(fileptr->data);
 #endif
@@ -378,23 +388,22 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
 
     /* Ensure we haven't wrapped around again! */
     if (search_last_line &&
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	((!ISSET(BACKWARDS_SEARCH) && current_x_find > begin_x) ||
 	(ISSET(BACKWARDS_SEARCH) && current_x_find < begin_x))
 #else
 	current_x_find > begin_x
 #endif
 	) {
-	if (can_display_wrap)
-	    not_found_msg(needle);
+	not_found_msg(needle);
 	return FALSE;
     }
 
     /* We've definitely found something. */
     openfile->current = fileptr;
     openfile->current_x = current_x_find;
-    openfile->current_y = current_y_find;
     openfile->placewewant = xplustabs();
+    openfile->current_y = current_y_find;
 
     /* needle_len holds the length of needle. */
     if (needle_len != NULL)
@@ -403,6 +412,8 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
     return TRUE;
 }
 
+/* Clear the flag indicating that a search reached the last line of the
+ * file.  We need to do this just before a new search. */
 void findnextstr_wrap_reset(void)
 {
     search_last_line = FALSE;
@@ -424,10 +435,10 @@ void do_search(void)
     i = search_init(FALSE, FALSE);
     if (i == -1)	/* Cancel, Go to Line, blank search string, or
 			 * regcomp() failed. */
-	search_abort();
+	search_replace_abort();
     else if (i == -2)	/* Replace. */
 	do_replace();
-#if !defined(NANO_SMALL) || defined(HAVE_REGEX_H)
+#if !defined(NANO_TINY) || defined(HAVE_REGEX_H)
     else if (i == 1)	/* Case Sensitive, Backwards, or Regexp search
 			 * toggle. */
 	do_search();
@@ -442,7 +453,7 @@ void do_search(void)
     else
 	last_search = mallocstrcpy(last_search, answer);
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     /* If answer is not "", add this search string to the search history
      * list. */
     if (answer[0] != '\0')
@@ -450,8 +461,11 @@ void do_search(void)
 #endif
 
     findnextstr_wrap_reset();
-    didfind = findnextstr(TRUE, FALSE, FALSE, openfile->current,
-	openfile->current_x, answer, NULL);
+    didfind = findnextstr(
+#ifndef DISABLE_SPELLER
+	FALSE,
+#endif
+	FALSE, openfile->current, openfile->current_x, answer, NULL);
 
     /* Check to see if there's only one occurrence of the string and
      * we're on it now. */
@@ -465,7 +479,11 @@ void do_search(void)
 	 * which case it's the only occurrence. */
 	if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp,
 		last_search)) {
-	    didfind = findnextstr(TRUE, FALSE, TRUE, openfile->current,
+	    didfind = findnextstr(
+#ifndef DISABLE_SPELLER
+		FALSE,
+#endif
+		TRUE, openfile->current,
 		openfile->current_x, answer, NULL);
 	    if (fileptr == openfile->current && fileptr_x ==
 		openfile->current_x && !didfind)
@@ -480,11 +498,11 @@ void do_search(void)
 
     openfile->placewewant = xplustabs();
     edit_redraw(fileptr, old_pww);
-    search_abort();
+    search_replace_abort();
 }
 
-#ifndef NANO_SMALL
-/* Search for the next string without prompting. */
+#ifndef NANO_TINY
+/* Search for the last string without prompting. */
 void do_research(void)
 {
     filestruct *fileptr = openfile->current;
@@ -499,7 +517,6 @@ void do_research(void)
     search_init_globals();
 
     if (last_search[0] != '\0') {
-
 #ifdef HAVE_REGEX_H
 	/* Since answer is "", use last_search! */
 	if (ISSET(USE_REGEXP) && regexp_init(last_search) == 0)
@@ -507,8 +524,12 @@ void do_research(void)
 #endif
 
 	findnextstr_wrap_reset();
-	didfind = findnextstr(TRUE, FALSE, FALSE, openfile->current,
-		openfile->current_x, last_search, NULL);
+	didfind = findnextstr(
+#ifndef DISABLE_SPELLER
+		FALSE,
+#endif
+		FALSE, openfile->current, openfile->current_x,
+		last_search, NULL);
 
 	/* Check to see if there's only one occurrence of the string and
 	 * we're on it now. */
@@ -522,9 +543,12 @@ void do_research(void)
 	     * found again, in which case it's the only occurrence. */
 	    if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp,
 		last_search)) {
-		didfind = findnextstr(TRUE, FALSE, TRUE,
-			openfile->current, openfile->current_x, answer,
-			NULL);
+		didfind = findnextstr(
+#ifndef DISABLE_SPELLER
+			FALSE,
+#endif
+			TRUE, openfile->current, openfile->current_x,
+			answer, NULL);
 		if (fileptr == openfile->current && fileptr_x ==
 			openfile->current_x && !didfind)
 		    statusbar(_("This is the only occurrence"));
@@ -540,16 +564,10 @@ void do_research(void)
 
     openfile->placewewant = xplustabs();
     edit_redraw(fileptr, old_pww);
-    search_abort();
+    search_replace_abort();
 }
 #endif
 
-void replace_abort(void)
-{
-    /* For now, we do the same thing as search_abort(). */
-    search_abort();
-}
-
 #ifdef HAVE_REGEX_H
 int replace_regexp(char *string, bool create)
 {
@@ -650,9 +668,12 @@ char *replace_line(const char *needle)
  * needle is the string to seek.  We replace it with answer.  Return -1
  * if needle isn't found, else the number of replacements performed.  If
  * canceled isn't NULL, set it to TRUE if we canceled. */
-ssize_t do_replace_loop(const char *needle, const filestruct
-	*real_current, size_t *real_current_x, bool wholewords, bool
-	*canceled)
+ssize_t do_replace_loop(
+#ifndef DISABLE_SPELLER
+	bool whole_word,
+#endif
+	bool *canceled, const filestruct *real_current, size_t
+	*real_current_x, const char *needle)
 {
     ssize_t numreplaced = -1;
     size_t match_len;
@@ -661,7 +682,7 @@ ssize_t do_replace_loop(const char *needle, const filestruct
     /* The starting-line match and bol/eol regex flags. */
     bool begin_line = FALSE, bol_or_eol = FALSE;
 #endif
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     bool old_mark_set = openfile->mark_set;
     filestruct *edittop_save = openfile->edittop, *top, *bot;
     size_t top_x, bot_x;
@@ -686,7 +707,10 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 	*canceled = FALSE;
 
     findnextstr_wrap_reset();
-    while (findnextstr(TRUE, wholewords,
+    while (findnextstr(
+#ifndef DISABLE_SPELLER
+	whole_word,
+#endif
 #ifdef HAVE_REGEX_H
 	/* We should find a bol and/or eol regex only once per line.  If
 	 * the bol_or_eol flag is set, it means that the last search
@@ -734,7 +758,7 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 
 	    do_replace_highlight(TRUE, exp_word);
 
-	    i = do_yesno(TRUE, _("Replace this instance?"));
+	    i = do_yesno_prompt(TRUE, _("Replace this instance?"));
 
 	    do_replace_highlight(FALSE, exp_word);
 
@@ -769,7 +793,7 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 	    length_change = strlen(copy) -
 		strlen(openfile->current->data);
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    /* If the mark was on and (mark_begin, mark_begin_x) was the
 	     * top of it, don't change mark_begin_x. */
 	    if (!old_mark_set || !right_side_up) {
@@ -797,7 +821,7 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 				match_len;
 		    *real_current_x += length_change;
 		}
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    }
 #endif
 
@@ -805,7 +829,7 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 	     * text, so searching will resume after the replacement
 	     * text.  Note that current_x might be set to (size_t)-1
 	     * here. */
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	    if (!ISSET(BACKWARDS_SEARCH))
 #endif
 		openfile->current_x += match_len + length_change - 1;
@@ -832,7 +856,7 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 	}
     }
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     if (old_mark_set) {
 	/* If the mark was on, unpartition the filestruct so that it
 	 * contains all the text again, set edittop back to what it was
@@ -844,8 +868,9 @@ ssize_t do_replace_loop(const char *needle, const filestruct
     }
 #endif
 
-    /* If text has been added to the magicline, make a new magicline. */
-    if (openfile->filebot->data[0] != '\0')
+    /* If the NO_NEWLINES flag isn't set, and text has been added to the
+     * magicline, make a new magicline. */
+    if (!ISSET(NO_NEWLINES) && openfile->filebot->data[0] != '\0')
 	new_magicline();
 
     return numreplaced;
@@ -861,14 +886,14 @@ void do_replace(void)
 
     if (ISSET(VIEW_MODE)) {
 	print_view_warning();
-	replace_abort();
+	search_replace_abort();
 	return;
     }
 
     i = search_init(TRUE, FALSE);
     if (i == -1) {		/* Cancel, Go to Line, blank search
 				 * string, or regcomp() failed. */
-	replace_abort();
+	search_replace_abort();
 	return;
     } else if (i == -2) {	/* No Replace. */
 	do_search();
@@ -883,7 +908,7 @@ void do_replace(void)
     /* If answer is not "", add answer to the search history list and
      * copy answer into last_search. */
     if (answer[0] != '\0') {
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	update_history(&search_history, answer);
 #endif
 	last_search = mallocstrcpy(last_search, answer);
@@ -891,13 +916,13 @@ void do_replace(void)
 
     last_replace = mallocstrcpy(last_replace, "");
 
-    i = statusq(FALSE, replace_list_2, last_replace,
-#ifndef NANO_SMALL
+    i = do_prompt(FALSE, replace_list_2, last_replace,
+#ifndef NANO_TINY
 	&replace_history,
 #endif
 	_("Replace with"));
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
     /* Add this replace string to the replace history list.  i == 0
      * means that the string is not "". */
     if (i == 0)
@@ -910,7 +935,7 @@ void do_replace(void)
 		answer = mallocstrcpy(answer, last_replace);
 	    statusbar(_("Cancelled"));
 	}
-	replace_abort();
+	search_replace_abort();
 	return;
     }
 
@@ -922,8 +947,11 @@ void do_replace(void)
     begin_x = openfile->current_x;
     pww_save = openfile->placewewant;
 
-    numreplaced = do_replace_loop(last_search, begin, &begin_x, FALSE,
-	NULL);
+    numreplaced = do_replace_loop(
+#ifndef DISABLE_SPELLER
+	FALSE,
+#endif
+	NULL, begin, &begin_x, last_search);
 
     /* Restore where we were. */
     openfile->edittop = edittop_save;
@@ -938,7 +966,7 @@ void do_replace(void)
 		"Replaced %lu occurrences", (unsigned long)numreplaced),
 		(unsigned long)numreplaced);
 
-    replace_abort();
+    search_replace_abort();
 }
 
 /* Go to the specified line and column, or ask for them if interactive
@@ -952,8 +980,8 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 	char *ans = mallocstrcpy(NULL, answer);
 
 	/* Ask for it. */
-	int i = statusq(FALSE, gotoline_list, use_answer ? ans : "",
-#ifndef NANO_SMALL
+	int i = do_prompt(FALSE, gotoline_list, use_answer ? ans : "",
+#ifndef NANO_TINY
 		NULL,
 #endif
 		_("Enter line number, column number"));
@@ -994,7 +1022,7 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
     }
 
     for (openfile->current = openfile->fileage;
-	openfile->current->next != NULL && line > 1; line--)
+	openfile->current != openfile->filebot && line > 1; line--)
 	openfile->current = openfile->current->next;
 
     openfile->current_x = actual_x(openfile->current->data, column - 1);
@@ -1012,6 +1040,7 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
     display_main_list();
 }
 
+/* Go to the specified line and column, asking for them beforehand. */
 void do_gotolinecolumn_void(void)
 {
     do_gotolinecolumn(openfile->current->lineno,
@@ -1019,13 +1048,16 @@ void do_gotolinecolumn_void(void)
 }
 
 #ifndef DISABLE_SPELLER
-void do_gotopos(ssize_t line, size_t pos_x, ssize_t pos_y, size_t
+/* Go to the line with the number specified in pos_line, the
+ * x-coordinate specified in pos_x, the y-coordinate specified in pos_y,
+ * and the place we want specified in pos_pww. */
+void do_gotopos(ssize_t pos_line, size_t pos_x, ssize_t pos_y, size_t
 	pos_pww)
 {
     /* Since do_gotolinecolumn() resets the x-coordinate but not the
      * y-coordinate, set the coordinates up this way. */
     openfile->current_y = pos_y;
-    do_gotolinecolumn(line, pos_x + 1, FALSE, FALSE, TRUE, TRUE);
+    do_gotolinecolumn(pos_line, pos_x + 1, FALSE, FALSE, TRUE, TRUE);
 
     /* Set the rest of the coordinates up. */
     openfile->placewewant = pos_pww;
@@ -1033,23 +1065,93 @@ void do_gotopos(ssize_t line, size_t pos_x, ssize_t pos_y, size_t
 }
 #endif
 
-#ifndef NANO_SMALL
-#ifdef HAVE_REGEX_H
+#ifndef NANO_TINY
+/* Search for a match to one of the two characters in bracket_set.  If
+ * reverse is TRUE, search backwards for the leftmost bracket.
+ * Otherwise, search forwards for the rightmost bracket.  Return TRUE if
+ * we found a match, and FALSE otherwise. */
+bool find_bracket_match(bool reverse, const char *bracket_set)
+{
+    filestruct *fileptr = openfile->current;
+    const char *rev_start = NULL, *found = NULL;
+    ssize_t current_y_find = openfile->current_y;
+
+    assert(strlen(bracket_set) == 2);
+
+    /* rev_start might end up 1 character before the start or after the
+     * end of the line.  This won't be a problem because we'll skip over
+     * it below in that case, and rev_start will be properly set when
+     * the search continues on the previous or next line. */
+    rev_start = reverse ? fileptr->data + (openfile->current_x - 1) :
+	fileptr->data + (openfile->current_x + 1);
+
+    /* Look for either of the two characters in bracket_set.  rev_start
+     * can be 1 character before the start or after the end of the line.
+     * In either case, just act as though no match is found. */
+    while (TRUE) {
+	found = ((rev_start > fileptr->data && *(rev_start - 1) ==
+		'\0') || rev_start < fileptr->data) ? NULL : (reverse ?
+		revstrpbrk(fileptr->data, bracket_set, rev_start) :
+		strpbrk(rev_start, bracket_set));
+
+	/* We've found a potential match. */
+	if (found != NULL)
+	    break;
+
+	if (reverse) {
+	    fileptr = fileptr->prev;
+	    current_y_find--;
+	} else {
+	    fileptr = fileptr->next;
+	    current_y_find++;
+	}
+
+	/* We've reached the start or end of the buffer, so get out. */
+	if (fileptr == NULL)
+	    return FALSE;
+
+	rev_start = fileptr->data;
+	if (reverse)
+	    rev_start += strlen(fileptr->data);
+    }
+
+    /* We've definitely found something. */
+    openfile->current = fileptr;
+    openfile->current_x = found - fileptr->data;
+    openfile->placewewant = xplustabs();
+    openfile->current_y = current_y_find;
+
+    return TRUE;
+}
+
+/* Search for a match to the bracket at the current cursor position, if
+ * there is one. */
 void do_find_bracket(void)
 {
-    const char *bracket_pat = "()<>[]{}", *pos;
-    char regex_pat[] = "[  ]", ch, wanted_ch;
     filestruct *current_save;
     size_t current_x_save, pww_save;
-    bool regexp_set = ISSET(USE_REGEXP);
-    bool backwards_search_set = ISSET(BACKWARDS_SEARCH);
-    int count = 1;
+    const char *bracket_list = "()<>[]{}";
+	/* The list of brackets we can find matches to. */
+    const char *pos;
+	/* The location in bracket_list of the bracket at the current
+	 * cursor position. */
+    char ch;
+	/* The bracket at the current cursor position. */
+    char wanted_ch;
+	/* The bracket complementing the bracket at the current cursor
+	 * position. */
+    char bracket_set[3];
+	/* The pair of characters in ch and wanted_ch. */
+    size_t count = 1;
+	/* The initial bracket count. */
+    bool reverse;
+	/* The direction we search. */
 
-    assert(strlen(bracket_pat) % 2 == 0);
+    assert(strlen(bracket_list) % 2 == 0);
 
     ch = openfile->current->data[openfile->current_x];
 
-    if (ch == '\0' || (pos = strchr(bracket_pat, ch)) == NULL) {
+    if (ch == '\0' || (pos = strchr(bracket_list, ch)) == NULL) {
 	statusbar(_("Not a bracket"));
 	return;
     }
@@ -1059,40 +1161,17 @@ void do_find_bracket(void)
     current_x_save = openfile->current_x;
     pww_save = openfile->placewewant;
 
-    /* Turn regular expression searches on. */
-    SET(USE_REGEXP);
-
     /* If we're on an opening bracket, we want to search forwards for a
      * closing bracket, and if we're on a closing bracket, we want to
-     * search backwards for an opening bracket.
-     *
-     * We replace the spaces in regex_pat with the opening and closing
-     * brackets, and then do a regular expression search using it.  We
-     * have to put the closing bracket first when we do the latter,
-     * since "[[]]" won't work properly, but "[][]" will. */
-    if ((pos - bracket_pat) % 2 == 0) {
-	wanted_ch = *(pos + 1);
-	regex_pat[1] = wanted_ch;
-	regex_pat[2] = ch;
-	UNSET(BACKWARDS_SEARCH);
-    } else {
-	wanted_ch = *(pos - 1);
-	regex_pat[1] = ch;
-	regex_pat[2] = wanted_ch;
-	SET(BACKWARDS_SEARCH);
-    }
-
-    /* Compile the regular expression in regex_pat. */
-    regexp_init(regex_pat);
-
-    /* The regular expression in regex_pat should always be valid. */
-    assert(regexp_compiled);
-
-    findnextstr_wrap_reset();
+     * search backwards for an opening bracket. */
+    reverse = ((pos - bracket_list) % 2 != 0);
+    wanted_ch = reverse ? *(pos - 1) : *(pos + 1);
+    bracket_set[0] = ch;
+    bracket_set[1] = wanted_ch;
+    bracket_set[2] = '\0';
 
     while (TRUE) {
-	if (findnextstr(FALSE, FALSE, FALSE, openfile->current,
-		openfile->current_x, regex_pat, NULL)) {
+	if (find_bracket_match(reverse, bracket_set)) {
 	    /* If we found an identical bracket, increment count.  If we
 	     * found a complementary bracket, decrement it. */
 	    count += (openfile->current->data[openfile->current_x] ==
@@ -1114,21 +1193,7 @@ void do_find_bracket(void)
 	    break;
 	}
     }
-
-    /* Decompile the regular expression in regex_pat. */
-    regexp_cleanup();
-
-    /* Restore search/replace direction. */
-    if (backwards_search_set)
-	SET(BACKWARDS_SEARCH);
-    else
-	UNSET(BACKWARDS_SEARCH);
-
-    /* Restore regular expression usage setting. */
-    if (!regexp_set)
-	UNSET(USE_REGEXP);
 }
-#endif /* HAVE_REGEX_H */
 
 #ifdef ENABLE_NANORC
 /* Indicate whether any of the history lists have changed. */
@@ -1325,4 +1390,4 @@ char *get_history_completion(filestruct **h, const char *s, size_t len)
     return (char *)s;
 }
 #endif /* !DISABLE_TABCOMP */
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
