@@ -1,8 +1,9 @@
-/* $Id: utils.c,v 1.93 2005/07/26 06:13:45 dolorous Exp $ */
+/* $Id: utils.c,v 1.110 2005/12/23 03:15:12 dolorous Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
- *   Copyright (C) 1999-2005 Chris Allegretta                             *
+ *   Copyright (C) 1999-2004 Chris Allegretta                             *
+ *   Copyright (C) 2005 David Lawrence Ramsey                             *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -20,9 +21,7 @@
  *                                                                        *
  **************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "proto.h"
 
 #include <string.h>
 #include <stdio.h>
@@ -30,8 +29,8 @@
 #include <pwd.h>
 #include <ctype.h>
 #include <errno.h>
-#include "proto.h"
 
+/* Return the number of decimal digits in n. */
 int digits(size_t n)
 {
     int i = 1;
@@ -95,7 +94,7 @@ bool parse_line_column(const char *str, ssize_t *line, ssize_t *column)
     comma = strchr(str, ',');
 
     if (comma != NULL && column != NULL) {
-	if (!parse_num(str + (comma - str + 1), column))
+	if (!parse_num(comma + 1, column))
 	    retval = FALSE;
     }
 
@@ -157,10 +156,33 @@ void sunder(char *str)
     }
 }
 
-#if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
+/* These functions, ngetline() (originally getline()) and ngetdelim()
+ * (originally getdelim()), were adapted from GNU mailutils 0.5
+ * (mailbox/getline.c).  Here is the notice from that file, after
+ * converting to the GPL via LGPL clause 3, and with the Free Software
+ * Foundation's address updated:
+ *
+ * GNU Mailutils -- a suite of utilities for electronic mail
+ * Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301, USA. */
+
+#if !defined(NANO_TINY) && defined(ENABLE_NANORC)
 #ifndef HAVE_GETLINE
-/* This function is equivalent to getline().  It was adapted from
- * GNU mailutils' getline() function. */
+/* This function is equivalent to getline(). */
 ssize_t ngetline(char **lineptr, size_t *n, FILE *stream)
 {
     return getdelim(lineptr, n, '\n', stream);
@@ -168,16 +190,14 @@ ssize_t ngetline(char **lineptr, size_t *n, FILE *stream)
 #endif
 
 #ifndef HAVE_GETDELIM
-/* This function is equivalent to getdelim().  It was adapted from
- * GNU mailutils' getdelim() function. */
+/* This function is equivalent to getdelim(). */
 ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
 {
     size_t indx = 0;
     int c;
 
     /* Sanity checks. */
-    if (lineptr == NULL || n == NULL || stream == NULL)
-	return -1;
+    assert(lineptr != NULL && n != NULL && stream != NULL);
 
     /* Allocate the line the first time. */
     if (*lineptr == NULL) {
@@ -192,7 +212,7 @@ ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
 	    *n += MAX_BUF_SIZE;
 	}
 
-	/* Push the result in the line. */
+	/* Put the result in the line. */
 	(*lineptr)[indx++] = (char)c;
 
 	/* Bail out. */
@@ -210,12 +230,12 @@ ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
     null_at(lineptr, indx++);
     *n = indx;
 
-    /* The last line may not have the delimiter, we have to return what
-     * we got and the error will be seen on the next iteration. */
+    /* The last line may not have the delimiter.  We have to return what
+     * we got, and the error will be seen on the next iteration. */
     return (c == EOF && (indx - 1) == 0) ? -1 : indx - 1;
 }
 #endif
-#endif /* !NANO_SMALL && ENABLE_NANORC */
+#endif /* !NANO_TINY && ENABLE_NANORC */
 
 #ifdef HAVE_REGEX_H
 #ifdef BROKEN_REGEXEC
@@ -230,7 +250,9 @@ int safe_regexec(const regex_t *preg, const char *string, size_t nmatch,
 }
 #endif
 
-int regexp_bol_or_eol(const regex_t *preg, const char *string)
+/* Do the compiled regex in preg and the regex in string match the
+ * beginning or end of a line? */
+bool regexp_bol_or_eol(const regex_t *preg, const char *string)
 {
     return (regexec(preg, string, 0, NULL, 0) == 0 &&
 	regexec(preg, string, 0, NULL, REG_NOTBOL | REG_NOTEOL) ==
@@ -238,6 +260,7 @@ int regexp_bol_or_eol(const regex_t *preg, const char *string)
 }
 #endif /* HAVE_REGEX_H */
 
+#ifndef DISABLE_SPELLER
 /* Is the word starting at position pos in buf a whole word? */
 bool is_whole_word(size_t pos, const char *buf, const char *word)
 {
@@ -262,6 +285,7 @@ bool is_whole_word(size_t pos, const char *buf, const char *word)
 
     return retval;
 }
+#endif /* !DISABLE_SPELLER */
 
 /* If we are searching backwards, we will find the last match that
  * starts no later than start.  Otherwise we find the first match
@@ -281,7 +305,7 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 
 #ifdef HAVE_REGEX_H
     if (ISSET(USE_REGEXP)) {
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	if (ISSET(BACKWARDS_SEARCH)) {
 	    if (regexec(&search_regexp, haystack, 1, regmatches,
 		0) == 0 && haystack + regmatches[0].rm_so <= start) {
@@ -299,7 +323,7 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 		return retval;
 	    }
 	} else
-#endif /* !NANO_SMALL */
+#endif /* !NANO_TINY */
 	if (regexec(&search_regexp, start, 10, regmatches,
 		(start > haystack) ? REG_NOTBOL : 0) == 0) {
 	    const char *retval = start + regmatches[0].rm_so;
@@ -310,17 +334,17 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 	return NULL;
     }
 #endif /* HAVE_REGEX_H */
-#if !defined(NANO_SMALL) || !defined(DISABLE_SPELLER)
+#if !defined(NANO_TINY) || !defined(DISABLE_SPELLER)
     if (ISSET(CASE_SENSITIVE)) {
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 	if (ISSET(BACKWARDS_SEARCH))
 	    return revstrstr(haystack, needle, start);
 	else
 #endif
 	    return strstr(start, needle);
     }
-#endif /* !DISABLE_SPELLER || !NANO_SMALL */
-#ifndef NANO_SMALL
+#endif /* !DISABLE_SPELLER || !NANO_TINY */
+#ifndef NANO_TINY
     else if (ISSET(BACKWARDS_SEARCH))
 	return mbrevstrcasestr(haystack, needle, start);
 #endif
@@ -340,7 +364,9 @@ void nperror(const char *s)
     }
 }
 
-/* Thanks, BG, many people have been asking for this... */
+/* This is a wrapper for the malloc() function that properly handles
+ * things when we run out of memory.  Thanks, BG, many people have been
+ * asking for this... */
 void *nmalloc(size_t howmuch)
 {
     void *r = malloc(howmuch);
@@ -351,6 +377,8 @@ void *nmalloc(size_t howmuch)
     return r;
 }
 
+/* This is a wrapper for the realloc() function that properly handles
+ * things when we run out of memory. */
 void *nrealloc(void *ptr, size_t howmuch)
 {
     void *r = realloc(ptr, howmuch);
@@ -395,6 +423,87 @@ char *mallocstrassn(char *dest, char *src)
     return src;
 }
 
+/* nano scrolls horizontally within a line in chunks.  Return the column
+ * number of the first character displayed in the edit window when the
+ * cursor is at the given column.  Note that (0 <= column -
+ * get_page_start(column) < COLS). */
+size_t get_page_start(size_t column)
+{
+    if (column == 0 || column < COLS - 1)
+	return 0;
+    else if (COLS > 8)
+	return column - 7 - (column - 7) % (COLS - 8);
+    else
+	return column - (COLS - 2);
+}
+
+/* Return the placewewant associated with current_x, i.e, the zero-based
+ * column position of the cursor.  The value will be no smaller than
+ * current_x. */
+size_t xplustabs(void)
+{
+    return strnlenpt(openfile->current->data, openfile->current_x);
+}
+
+/* Return the index in s of the character displayed at the given column,
+ * i.e, the largest value such that strnlenpt(s, actual_x(s, column)) <=
+ * column. */
+size_t actual_x(const char *s, size_t column)
+{
+    size_t i = 0;
+	/* The position in s, returned. */
+    size_t len = 0;
+	/* The screen display width to s[i]. */
+
+    assert(s != NULL);
+
+    while (*s != '\0') {
+	int s_len = parse_mbchar(s, NULL, &len);
+
+	if (len > column)
+	    break;
+
+	i += s_len;
+	s += s_len;
+    }
+
+    return i;
+}
+
+/* A strnlen() with tabs and multicolumn characters factored in, similar
+ * to xplustabs().  How many columns wide are the first maxlen characters
+ * of s? */
+size_t strnlenpt(const char *s, size_t maxlen)
+{
+    size_t len = 0;
+	/* The screen display width to s[i]. */
+
+    if (maxlen == 0)
+	return 0;
+
+    assert(s != NULL);
+
+    while (*s != '\0') {
+	int s_len = parse_mbchar(s, NULL, &len);
+
+	s += s_len;
+
+	if (maxlen <= s_len)
+	    break;
+
+	maxlen -= s_len;
+    }
+
+    return len;
+}
+
+/* A strlen() with tabs and multicolumn characters factored in, similar
+ * to xplustabs().  How many columns wide is s? */
+size_t strlenpt(const char *s)
+{
+    return strnlenpt(s, (size_t)-1);
+}
+
 /* Append a new magicline to filebot. */
 void new_magicline(void)
 {
@@ -407,13 +516,16 @@ void new_magicline(void)
     openfile->totsize++;
 }
 
-#ifndef NANO_SMALL
+#ifndef NANO_TINY
 /* Remove the magicline from filebot, if there is one and it isn't the
- * only line in the file. */
+ * only line in the file.  Assume that edittop and current are not at
+ * filebot. */
 void remove_magicline(void)
 {
     if (openfile->filebot->data[0] == '\0' &&
-	openfile->filebot->prev != NULL) {
+	openfile->filebot != openfile->fileage) {
+	assert(openfile->filebot != openfile->edittop && openfile->filebot != openfile->current);
+
 	openfile->filebot = openfile->filebot->prev;
 	free_filestruct(openfile->filebot->next);
 	openfile->filebot->next = NULL;
@@ -480,3 +592,49 @@ size_t get_totsize(const filestruct *begin, const filestruct *end)
 
     return totsize;
 }
+
+#ifndef NDEBUG
+/* Return what the current line number should be, starting at edittop
+ * and ending at fileptr. */
+int check_linenumbers(const filestruct *fileptr)
+{
+    int check_line = 0;
+    const filestruct *filetmp;
+
+    for (filetmp = openfile->edittop; filetmp != fileptr;
+	filetmp = filetmp->next)
+	check_line++;
+
+    return check_line;
+}
+#endif /* !NDEBUG */
+
+#ifdef DEBUG
+/* Dump the filestruct inptr to stderr. */
+void dump_filestruct(const filestruct *inptr)
+{
+    if (inptr == openfile->fileage)
+	fprintf(stderr, "Dumping file buffer to stderr...\n");
+    else if (inptr == cutbuffer)
+	fprintf(stderr, "Dumping cutbuffer to stderr...\n");
+    else
+	fprintf(stderr, "Dumping a buffer to stderr...\n");
+
+    while (inptr != NULL) {
+	fprintf(stderr, "(%ld) %s\n", (long)inptr->lineno, inptr->data);
+	inptr = inptr->next;
+    }
+}
+
+/* Dump the current buffer's filestruct to stderr in reverse. */
+void dump_filestruct_reverse(void)
+{
+    const filestruct *fileptr = openfile->filebot;
+
+    while (fileptr != NULL) {
+	fprintf(stderr, "(%ld) %s\n", (long)fileptr->lineno,
+		fileptr->data);
+	fileptr = fileptr->prev;
+    }
+}
+#endif /* DEBUG */
